@@ -101,4 +101,10 @@ as_status as_batch_read_results_to_pyobject(as_error* err, AerospikeClient* clie
 as_status batch_read_records_to_pyobject(AerospikeClient *self, as_error *err, as_batch_read_records* records, PyObject **py_recs);
 
 as_status
-string_and_pyuni_from_pystring(PyObject* py_string, PyObject** pyuni_r, char** c_str_ptr, as_error* err);
\ No newline at end of file
+string_and_pyuni_from_pystring(PyObject* py_string, PyObject** pyuni_r, char** c_str_ptr, as_error* err);
+
+as_status
+get_cdt_ctx(AerospikeClient* self, as_error* err, as_cdt_ctx* cdt_ctx, PyObject* op_dict, bool* ctx_in_use, as_static_pool* static_pool, int serializer_type);
+
+as_status 
+convert_predexp_list(PyObject* py_predexp_list, as_predexp_list* predexp_list, as_error* err);
