@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2017 Aerospike, Inc.
+ * Copyright 2013-2019 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -28,7 +28,11 @@
 	#define PyString_FromString         PyUnicode_FromString
 	#define PyString_FromStringAndSize  PyUnicode_FromStringAndSize
 
-	#define PyString_AsString           PyUnicode_AsUTF8
+	#if PY_MINOR_VERSION < 7
+			#define PyString_AsString           PyUnicode_AsUTF8
+	#else
+			#define PyString_AsString           (char*)PyUnicode_AsUTF8
+	#endif
 
 	#define PyString_Size               PyUnicode_GET_SIZE
 	#define PyString_GET_SIZE           PyUnicode_GET_SIZE
@@ -49,3 +53,6 @@
 			ob = Py_InitModule3(name, methods, doc);
 	#define MOD_SUCCESS_VAL(val)
 #endif
+
+// pyval is a PyObject* classname is a string
+#define AS_Matches_Classname(pyval, classname) (strcmp((pyval)->ob_type->tp_name, (classname)) == 0)
\ No newline at end of file
