@@ -21,6 +21,8 @@
 #include <aerospike/as_policy.h>
 #include <aerospike/as_map_operations.h>
 #include <aerospike/as_list_operations.h>
+#include <aerospike/as_bit_operations.h>
+#include <aerospike/as_hll_operations.h>
 
 #define MAX_CONSTANT_STR_SIZE 512
 
@@ -107,6 +109,41 @@ enum Aerospike_map_operations {
 	OP_MAP_GET_BY_KEY_INDEX_RANGE_REL
 };
 
+enum aerospike_bitwise_operations {
+    OP_BIT_RESIZE = 2000,
+    OP_BIT_INSERT,
+    OP_BIT_REMOVE,
+    OP_BIT_SET,
+    OP_BIT_OR,
+    OP_BIT_XOR,
+    OP_BIT_AND,
+    OP_BIT_NOT,
+    OP_BIT_LSHIFT,
+    OP_BIT_RSHIFT,
+    OP_BIT_ADD,
+    OP_BIT_SUBTRACT,
+    OP_BIT_GET_INT,
+    OP_BIT_SET_INT,
+    OP_BIT_GET,
+    OP_BIT_COUNT,
+    OP_BIT_LSCAN,
+    OP_BIT_RSCAN
+};
+
+enum aerospike_hll_operations {
+	OP_HLL_ADD = 2100,
+	OP_HLL_DESCRIBE,
+	OP_HLL_FOLD,
+	OP_HLL_GET_COUNT,
+	OP_HLL_GET_INTERSECT_COUNT,
+	OP_HLL_GET_SIMILARITY,
+	OP_HLL_GET_UNION,
+	OP_HLL_GET_UNION_COUNT,
+	OP_HLL_INIT,
+	OP_HLL_REFRESH_COUNT,
+	OP_HLL_SET_UNION,
+};
+
 typedef struct Aerospike_Constants {
     long    constantno;
     char    constant_str[MAX_CONSTANT_STR_SIZE];
@@ -127,7 +164,9 @@ as_status pyobject_to_policy_admin(as_error * err, PyObject * py_policy,
 as_status pyobject_to_policy_apply(as_error * err, PyObject * py_policy,
 									as_policy_apply * policy,
 									as_policy_apply ** policy_p,
-									as_policy_apply * config_apply_policy);
+									as_policy_apply * config_apply_policy,
+									as_predexp_list * predexp_list,
+									as_predexp_list ** predexp_list_p);
 
 as_status pyobject_to_policy_info(as_error * err, PyObject * py_policy,
 									as_policy_info * policy,
@@ -137,37 +176,51 @@ as_status pyobject_to_policy_info(as_error * err, PyObject * py_policy,
 as_status pyobject_to_policy_query(as_error * err, PyObject * py_policy,
 									as_policy_query * policy,
 									as_policy_query ** policy_p,
-									as_policy_query * config_query_policy);
+									as_policy_query * config_query_policy,
+									as_predexp_list * predexp_list,
+									as_predexp_list ** predexp_list_p);
 
 as_status pyobject_to_policy_read(as_error * err, PyObject * py_policy,
 									as_policy_read * policy,
 									as_policy_read ** policy_p,
-									as_policy_read * config_read_policy);
+									as_policy_read * config_read_policy,
+									as_predexp_list * predexp_list,
+									as_predexp_list ** predexp_list_p);
 
 as_status pyobject_to_policy_remove(as_error * err, PyObject * py_policy,
 									as_policy_remove * policy,
 									as_policy_remove ** policy_p,
-									as_policy_remove * config_remove_policy);
+									as_policy_remove * config_remove_policy,
+									as_predexp_list * predexp_list,
+									as_predexp_list ** predexp_list_p);
 
 as_status pyobject_to_policy_scan(as_error * err, PyObject * py_policy,
 									as_policy_scan * policy,
 									as_policy_scan ** policy_p,
-									as_policy_scan * config_scan_policy);
+									as_policy_scan * config_scan_policy,
+									as_predexp_list * predexp_list,
+									as_predexp_list ** predexp_list_p);
 
 as_status pyobject_to_policy_write(as_error * err, PyObject * py_policy,
 									as_policy_write * policy,
 									as_policy_write ** policy_p,
-									as_policy_write * config_write_policy);
+									as_policy_write * config_write_policy,
+									as_predexp_list * predexp_list,
+									as_predexp_list ** predexp_list_p);
 
 as_status pyobject_to_policy_operate(as_error * err, PyObject * py_policy,
                                     as_policy_operate * policy,
                                     as_policy_operate ** policy_p,
-									as_policy_operate * config_operate_policy);
+									as_policy_operate * config_operate_policy,
+									as_predexp_list * predexp_list,
+									as_predexp_list ** predexp_list_p);
 
 as_status pyobject_to_policy_batch(as_error * err, PyObject * py_policy,
                                    as_policy_batch * policy,
                                    as_policy_batch ** policy_p,
-								   as_policy_batch * config_batch_policy);
+								   as_policy_batch * config_batch_policy,
+								   as_predexp_list * predexp_list,
+								   as_predexp_list ** predexp_list_p);
 
 as_status pyobject_to_map_policy(as_error * err, PyObject * py_policy,
 									as_map_policy * policy);
@@ -181,3 +234,6 @@ as_status set_query_options(as_error* err, PyObject* query_options, as_query* qu
 as_status pyobject_to_list_policy(as_error * err, PyObject * py_policy,
 									as_list_policy * policy);
 
+as_status pyobject_to_bit_policy(as_error* err, PyObject* py_policy, as_bit_policy* policy);
+
+as_status pyobject_to_hll_policy(as_error* err, PyObject* py_policy, as_hll_policy* hll_policy);
