@@ -31,9 +31,10 @@
 #include "serializer.h"
 #include "module_functions.h"
 #include "nullobject.h"
+#include "cdt_types.h"
 
 PyObject *py_global_hosts;
-int counter = 0xA7000000;
+int counter = 0xA8000000;
 bool user_shm_key = false;
 
 PyDoc_STRVAR(client_doc,
@@ -86,14 +87,15 @@ AerospikeConstants operator_constants[] = {
 	{ AS_OPERATOR_INCR                 ,   "OPERATOR_INCR"    },
 	{ AS_OPERATOR_APPEND               ,   "OPERATOR_APPEND"  },
 	{ AS_OPERATOR_PREPEND              ,   "OPERATOR_PREPEND" },
-	{ AS_OPERATOR_TOUCH                ,   "OPERATOR_TOUCH"   }
+	{ AS_OPERATOR_TOUCH                ,   "OPERATOR_TOUCH"   },
+	{ AS_OPERATOR_DELETE               ,   "OPERATOR_DELETE"  }
 };
 
 #define OPERATOR_CONSTANTS_ARR_SIZE (sizeof(operator_constants)/sizeof(AerospikeConstants))
 MOD_INIT(aerospike)
 {
 
-	const char version[8] = "3.4.1";
+	const char version[8] = "4.0.0";
 	// Makes things "thread-safe"
 	PyEval_InitThreads();
 	int i = 0;
@@ -151,5 +153,13 @@ MOD_INIT(aerospike)
 	Py_INCREF(null_object);
 	PyModule_AddObject(aerospike, "null", (PyObject *) null_object);
 
+	PyTypeObject * wildcard_object = AerospikeWildcardObject_Ready();
+	Py_INCREF(wildcard_object);
+	PyModule_AddObject(aerospike, "CDTWildcard", (PyObject *) wildcard_object);
+
+	PyTypeObject * infinite_object = AerospikeInfiniteObject_Ready();
+	Py_INCREF(infinite_object);
+	PyModule_AddObject(aerospike, "CDTInfinite", (PyObject *) infinite_object);
+
 	return MOD_SUCCESS_VAL(aerospike);
 }
