@@ -62,6 +62,10 @@ PyObject * AerospikeClient_Apply_Invoke(
 	PyObject * py_umodule   = NULL;
 	PyObject * py_ufunction = NULL;
 
+	// For converting predexp.
+	as_predexp_list predexp_list;
+	as_predexp_list* predexp_list_p = NULL;
+
 	as_static_pool static_pool;
 	memset(&static_pool, 0, sizeof(static_pool));
 	// Initialisation flags
@@ -102,7 +106,7 @@ PyObject * AerospikeClient_Apply_Invoke(
 
 	// Convert python policy object to as_policy_apply
 	pyobject_to_policy_apply(&err, py_policy, &apply_policy, &apply_policy_p,
-			&self->as->config.policies.apply);
+			&self->as->config.policies.apply, &predexp_list, &predexp_list_p);
 	if (err.code != AEROSPIKE_OK) {
 		goto CLEANUP;
 	}
@@ -144,6 +148,10 @@ PyObject * AerospikeClient_Apply_Invoke(
 
 CLEANUP:
 
+	if (predexp_list_p) {
+		as_predexp_list_destroy(&predexp_list);
+	}
+
 	if (py_umodule) {
 		Py_DECREF(py_umodule);
 	}
