@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2017 Aerospike, Inc.
+ * Copyright 2013-2020 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -54,6 +54,10 @@ PyObject * AerospikeClient_Get_Invoke(
 	as_key key;
 	as_record * rec = NULL;
 
+	// For converting predexp.
+	as_predexp_list predexp_list;
+	as_predexp_list* predexp_list_p = NULL;
+
 	// Initialised flags
 	bool key_initialised = false;
 	bool record_initialised = false;
@@ -81,21 +85,18 @@ PyObject * AerospikeClient_Get_Invoke(
 
 	// Convert python policy object to as_policy_exists
 	pyobject_to_policy_read(&err, py_policy, &read_policy, &read_policy_p,
-			&self->as->config.policies.read);
+			&self->as->config.policies.read, &predexp_list, &predexp_list_p);
 	if (err.code != AEROSPIKE_OK) {
 		goto CLEANUP;
 	}
 
-	// Initialize record
-	as_record_init(rec, 0);
-	// Record initialised successfully.
-	record_initialised = true;
-
 	// Invoke operation
 	Py_BEGIN_ALLOW_THREADS
 	aerospike_key_get(self->as, &err, read_policy_p, &key, &rec);
 	Py_END_ALLOW_THREADS
 	if (err.code == AEROSPIKE_OK) {
+		record_initialised = true;
+
 		if (record_to_pyobject(self, &err, rec, &key, &py_rec) != AEROSPIKE_OK) {
 			goto CLEANUP;
 		}
@@ -117,11 +118,16 @@ PyObject * AerospikeClient_Get_Invoke(
 
 CLEANUP:
 
+	if (predexp_list_p) {
+		as_predexp_list_destroy(&predexp_list);
+	}
+
 	if (key_initialised == true) {
 		// Destroy key only if it is initialised.
 		as_key_destroy(&key);
 	}
-	if (record_initialised == true) {
+
+	if (rec && record_initialised) {
 		// Destroy record only if it is initialised.
 		as_record_destroy(rec);
 	}
