@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2016 Aerospike, Inc.
+ * Copyright 2013-2017 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,12 +22,15 @@
 #include <aerospike/as_error.h>
 #include <aerospike/as_node.h>
 #include <aerospike/as_record.h>
+#include <aerospike/as_cluster.h>
 
 #include "client.h"
 #include "conversions.h"
 #include "exceptions.h"
 #include "policy.h"
+#include "tls_info_host.h"
 
+static char* get_unbracketed_ip_and_length(char* ip_start, char* split_point, int* length);
 /**
  ********************************************************************************************************
  * Macros for Info API.
@@ -57,10 +60,13 @@ static PyObject * AerospikeClient_InfoNode_Invoke(
 	PyObject * py_response = NULL;
 	PyObject * py_ustr = NULL;
 	PyObject * py_ustr1 = NULL;
+	PyObject* py_uni_tls_name = NULL;
+
 	as_policy_info info_policy;
 	as_policy_info* info_policy_p = NULL;
 	as_host * host = NULL;
-	char* address =NULL;
+	char* address = NULL;
+	char* tls_name = NULL;
 	long port_no;
 	char* response_p = NULL;
 	as_status status = AEROSPIKE_OK;
@@ -85,20 +91,14 @@ static PyObject * AerospikeClient_InfoNode_Invoke(
 	port_no = host->port;
 
 	if (py_policy) {
-		if (PyDict_Check(py_policy)) {
-			pyobject_to_policy_info(err, py_policy, &info_policy, &info_policy_p,
-					&self->as->config.policies.info);
-			if (err->code != AEROSPIKE_OK) {
-				goto CLEANUP;
-			}
-		} else {
-			as_error_update(err, AEROSPIKE_ERR_PARAM, "Policy should be a dictionary");
+		if (pyobject_to_policy_info(err, py_policy, &info_policy, &info_policy_p,
+				&self->as->config.policies.info) != AEROSPIKE_OK) {
 			goto CLEANUP;
 		}
 	}
 
 	if (py_host) {
-		if (PyTuple_Check(py_host) && PyTuple_Size(py_host) == 2) {
+		if (PyTuple_Check(py_host) && ((PyTuple_Size(py_host) == 2) || (PyTuple_Size(py_host) == 3))) {
 			PyObject * py_addr = PyTuple_GetItem(py_host,0);
 			PyObject * py_port = PyTuple_GetItem(py_host,1);
 
@@ -114,6 +114,26 @@ static PyObject * AerospikeClient_InfoNode_Invoke(
 			else if (PyLong_Check(py_port)) {
 				port_no = (uint16_t) PyLong_AsLong(py_port);
 			}
+
+			/* handle TLS_NAME if present */
+			if (PyTuple_Size(py_host) == 3) {
+				PyObject * py_tls = PyTuple_GetItem(py_host, 2);
+
+				if (PyString_Check(py_tls)) {
+					tls_name = PyString_AsString(py_tls);
+				} else if (PyUnicode_Check(py_tls)) {
+					py_uni_tls_name = PyUnicode_AsUTF8String(py_tls);
+					if (!py_uni_tls_name) {
+						as_error_update(err, AEROSPIKE_ERR_PARAM, "Invalid unicode value");
+						goto CLEANUP;
+					}
+					tls_name = PyBytes_AsString(py_uni_tls_name);
+				} else {
+					as_error_update(err, AEROSPIKE_ERR_PARAM, "tls name must be string or unicode");
+					goto CLEANUP;
+				}
+			}
+
 		} else if (!PyTuple_Check(py_host)) {
 			as_error_update(err, AEROSPIKE_ERR_PARAM, "Host should be a specified in form of Tuple.");
 			goto CLEANUP;
@@ -132,9 +152,15 @@ static PyObject * AerospikeClient_InfoNode_Invoke(
 	}
 
 	Py_BEGIN_ALLOW_THREADS
-	status = aerospike_info_host(self->as, err, info_policy_p,
-		(const char *) address, (uint16_t) port_no, request_str_p,
-		&response_p);
+	if (!tls_name) {
+		status = aerospike_info_host(self->as, err, info_policy_p,
+			(const char *) address, (uint16_t) port_no, request_str_p,
+			&response_p);
+	} else {
+		/* Using tls, need to do the slow path */
+		status = send_info_to_tls_host(self->as, err, info_policy_p, address, port_no, tls_name,
+									   (const char *) request_str_p, &response_p);
+	}
 	Py_END_ALLOW_THREADS
 	if (err->code == AEROSPIKE_OK) {
 		if (response_p && status == AEROSPIKE_OK) {
@@ -148,7 +174,7 @@ static PyObject * AerospikeClient_InfoNode_Invoke(
 			goto CLEANUP;
 		}
 	} else {
-		as_error_update(err, err->code, err->message);
+		as_error_update(err, err->code, NULL);
 		goto CLEANUP;
 	}
 
@@ -160,6 +186,9 @@ CLEANUP:
 	if (py_ustr1) {
 		Py_DECREF(py_ustr1);
 	}
+	if (py_uni_tls_name) {
+		Py_DECREF(py_uni_tls_name);
+	}
 
 	if (err->code != AEROSPIKE_OK) {
 		PyObject * py_err = NULL;
@@ -203,170 +232,118 @@ PyObject * AerospikeClient_InfoNode(AerospikeClient * self, PyObject * args, PyO
 	return AerospikeClient_InfoNode_Invoke(&err, self, py_request, py_host, py_policy);
 
 }
-/**
- ******************************************************************************************************
- * Iterates over the hosts in the cluster and creates the list to be returned to the python client.
- *
- * @param err                   as_error object
- * @param command               Request string sent from the python client
- * @param nodes_tuple           List containing details of each host
- * @param return_value          List t o be returned back to the python client
- * @param host_index            Index of the list nodes_tuple
- * @param index                 Index of the list to be returned.
- *
- * Returns information about a host.
- ********************************************************************************************************/
-static PyObject * AerospikeClient_GetNodes_Returnlist(as_error* err,
-	PyObject * command, PyObject * nodes_tuple[], PyObject * return_value,
-	uint32_t host_index, Py_ssize_t index) {
 
-	char* tok = NULL;
-	char* saved = NULL;
-	PyObject * value_tok = NULL;
-	bool break_flag = false;
+	/**
+	 ******************************************************************************************************
+	 * Returns data about the nodes to AerospikeClient_GetNodes.
+	 *
+	 * @param self                  AerospikeClient object
+	 *
+	 * Returns a list containing the details of the nodes.
+	 ********************************************************************************************************/
+	static PyObject * AerospikeClient_GetNodes_Invoke(
+		AerospikeClient * self) {
 
-	tok = strtok_r(PyString_AsString(command), INFO_REQUEST_RESPONSE_DELIMITER, &saved);
-	if (!tok) {
-		as_error_update(err, AEROSPIKE_ERR_CLIENT, "Unable to get addr in service");
-		goto CLEANUP;
-	}
-	while (tok && (host_index < MAX_HOST_COUNT)) {
-		tok = strtok_r(NULL, IP_PORT_DELIMITER, &saved);
-#if defined(__APPLE__)
-		if (!tok || !saved) {
-#else
-		if (!tok || *saved == '\0') {
-#endif
+		PyObject* py_hostname = NULL;
+		PyObject* py_port = NULL;
+		PyObject* return_value = PyList_New(0);
+
+		as_nodes* nodes = NULL;
+		char* hostname = NULL;
+		char* split_point = NULL;
+
+		as_error err;
+		as_error_init(&err);
+
+		if (!self || !self->as) {
+			as_error_update(&err, AEROSPIKE_ERR_PARAM, "Invalid aerospike object");
 			goto CLEANUP;
 		}
 
-		nodes_tuple[host_index] = PyTuple_New(2);
-
-		value_tok = PyString_FromString(tok);
-		PyTuple_SetItem(nodes_tuple[host_index], 0 , value_tok);
-		//Py_DECREF(value_tok);
-
-		if (strcmp(PyString_AsString(command),"response_services_p")) {
-			tok = strtok_r(NULL, HOST_DELIMITER, &saved);
-			if (!tok) {
-				as_error_update(err, AEROSPIKE_ERR_CLIENT, "Unable to get port");
-				goto CLEANUP;
-			}
-
-			if (strstr(tok, INFO_RESPONSE_END)) {
-				tok = strtok_r(tok, INFO_RESPONSE_END, &saved);
-				break_flag = true;
-			}
-		} else {
-			tok = strtok_r(NULL, INFO_RESPONSE_END, &saved);
-			if (!tok) {
-				as_error_update(err, AEROSPIKE_ERR_CLIENT, "Unable to get port in service");
-				goto CLEANUP;
-			}
-		}
-
-		value_tok = PyInt_FromString(tok, NULL, 10);
-		PyTuple_SetItem(nodes_tuple[host_index], 1 , value_tok);
-		PyList_Insert(return_value, index , nodes_tuple[host_index]);
-		Py_DECREF(nodes_tuple[host_index]);
-		index++;
-		host_index++;
-
-		if (break_flag) {
+		if (!self->is_conn_16) {
+			as_error_update(&err, AEROSPIKE_ERR_CLUSTER, "No connection to aerospike cluster");
 			goto CLEANUP;
 		}
 
-	}
-CLEANUP:
-
-	if (err->code != AEROSPIKE_OK) {
-		PyObject * py_err = NULL;
-		error_to_pyobject(err, &py_err);
-		PyObject *exception_type = raise_exception(err);
-		PyErr_SetObject(exception_type, py_err);
-		Py_DECREF(py_err);
-		return NULL;
-	}
-	return return_value;
-}
-/**
- ******************************************************************************************************
- * Returns data about the nodes to AerospikeClient_GetNodes.
- *
- * @param self                  AerospikeClient object
- *
- * Returns a list containing the details of the nodes.
- ********************************************************************************************************/
-static PyObject * AerospikeClient_GetNodes_Invoke(
-	AerospikeClient * self) {
-
-	PyObject * response_services_p = NULL;
-	PyObject * response_service_p = NULL;
-	PyObject * nodes_tuple[MAX_HOST_COUNT] = {0};
-	PyObject * return_value = PyList_New(0);
-
-	as_error err;
-	as_error_init(&err);
-
-	if (!self || !self->as) {
-		as_error_update(&err, AEROSPIKE_ERR_PARAM, "Invalid aerospike object");
-		goto CLEANUP;
-	}
-
-	if (!self->is_conn_16) {
-		as_error_update(&err, AEROSPIKE_ERR_CLUSTER, "No connection to aerospike cluster");
-		goto CLEANUP;
-	}
-
-	PyObject * py_req_str = NULL;
-	py_req_str = PyString_FromString("services");
-	response_services_p = AerospikeClient_InfoNode_Invoke(&err, self, py_req_str, NULL, NULL);
-	Py_DECREF(py_req_str);
-	if (!response_services_p) {
-		if (err.code == AEROSPIKE_OK) {
-			as_error_update(&err, AEROSPIKE_ERR_CLIENT, "Services call returned an error");
+		aerospike* as = self->as;
+		as_cluster* cluster = as->cluster;
+		// If the cluster goes down between the last call and this call, this could theoretically occur.
+		if (!cluster) {
+			as_error_update(&err, AEROSPIKE_ERR_CLUSTER, "invalid aerospike cluster");
+			goto CLEANUP;
 		}
-		goto CLEANUP;
-	}
 
-	py_req_str = PyString_FromString("service");
-	response_service_p = AerospikeClient_InfoNode_Invoke(&err, self, py_req_str, NULL, NULL);
-	Py_DECREF(py_req_str);
-	if (!response_service_p) {
-		if (err.code == AEROSPIKE_OK) {
-			as_error_update(&err, AEROSPIKE_ERR_CLIENT, "Service call returned an error");
+		nodes = as_nodes_reserve(cluster);
+
+		if (!nodes) {
+			as_error_update(&err, AEROSPIKE_ERR_CLUSTER, "Cluster is empty");
+			goto CLEANUP;
 		}
-		goto CLEANUP;
+
+		for (uint32_t i = 0; i < nodes->size; i++) {
+			as_node* node = nodes->array[i];
+			hostname = (char*)as_node_get_address_string(node);
+			split_point = strrchr(hostname, ':');
+			if (!split_point) {
+				as_error_update(&err, AEROSPIKE_ERR_CLIENT, "Malformed host name string");
+				goto CLEANUP;
+			}
+
+			char* real_hostname_start = NULL;
+			int real_length;
+			/* Since hostname might be '[::1]' and we want to return only '::1' , we need to set
+			 * a pointer
+			 */
+			real_hostname_start = get_unbracketed_ip_and_length(hostname, split_point, &real_length);
+			Py_ssize_t py_host_length = (Py_ssize_t)real_length;
+			py_hostname = PyString_FromStringAndSize(real_hostname_start, py_host_length);
+
+			if (!py_hostname) {
+				as_error_update(&err, AEROSPIKE_ERR_CLIENT, "Failed to create python hostname");
+				goto CLEANUP;
+			}
+
+			// convert "3000" -> 3000, using base 10 | use long since it works in 2 & 3
+			py_port = PyLong_FromString(split_point + 1, NULL, 10);
+			if (!py_port || PyErr_Occurred()) {
+				// py_port exists
+				Py_XDECREF(py_hostname);
+				as_error_update(&err, AEROSPIKE_ERR_CLIENT, "Non numeric port found");
+				goto CLEANUP;
+			}
+			PyObject* py_host_pair = Py_BuildValue("OO", py_hostname, py_port);
+
+			Py_XDECREF(py_port);
+			Py_XDECREF(py_hostname);
+
+			if(!py_host_pair) {
+				as_error_update(&err, AEROSPIKE_ERR_CLIENT, "Failed to build node info tuple");
+				goto CLEANUP;
+			}
+
+			PyList_Append(return_value, py_host_pair);
+			Py_DECREF(py_host_pair);
+		}
+	CLEANUP:
+		if(nodes) {
+			as_nodes_release(nodes);
+		}
+		if (err.code != AEROSPIKE_OK) {
+			// Clear the return value if it exists
+			Py_XDECREF(return_value);
+			PyObject * py_err = NULL;
+			error_to_pyobject(&err, &py_err);
+			PyObject *exception_type = raise_exception(&err);
+			PyErr_SetObject(exception_type, py_err);
+			Py_DECREF(py_err);
+
+			return NULL;
+		}
+
+		return return_value;
 	}
 
-	return_value = AerospikeClient_GetNodes_Returnlist(&err, response_service_p, nodes_tuple, return_value, 0, 0);
-	if (return_value)
-		return_value = AerospikeClient_GetNodes_Returnlist(&err, response_services_p, nodes_tuple, return_value, 1, 1);
-
-CLEANUP:
-
-	if (response_services_p) {
-		Py_DECREF(response_services_p);
-	}
-
-	if (response_service_p) {
-		Py_DECREF(response_service_p);
-	}
-
-	if (err.code != AEROSPIKE_OK) {
-		PyObject * py_err = NULL;
-		error_to_pyobject(&err, &py_err);
-		PyObject *exception_type = raise_exception(&err);
-		PyErr_SetObject(exception_type, py_err);
-		Py_DECREF(py_err);
-		return NULL;
-	}
-
-	return return_value;
-}
-
-/**
- ******************************************************************************************************
+/******************************************************************************************************
  * Returns data about the nodes in a cluster of the database.
  *
  * @param self                  AerospikeClient object
@@ -380,3 +357,24 @@ PyObject * AerospikeClient_GetNodes(AerospikeClient * self, PyObject * args, PyO
 {
 	return AerospikeClient_GetNodes_Invoke(self);
 }
+
+/*
+ * This will return the correct starting point of an ipv6 address wrapped in [] and the length of the actual ip
+ * split_point is the address of the rightmost : in the ip_start string
+ * so something like ip_start = "[::1]:3000"
+ * where ip_start is 0xf0 and split_point is 0xf5
+ * len will be set to 3 and the return value will be (char*)0xf1
+ *
+ * If this is ipv4, length is the length of the address, and returned value = ip_start
+ */
+static char* get_unbracketed_ip_and_length(char* ip_start, char* split_point, int* length) {
+	*length = split_point - ip_start;
+	if (*length < 2) {
+		return ip_start;
+	}
+	if (ip_start[0] == '[' && ip_start[*length - 1] == ']') {
+		*length = *length - 2;
+		return (char*)(ip_start + 1);
+	}
+	return ip_start;
+}
