@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2017 Aerospike, Inc.
+ * Copyright 2013-2019 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -33,6 +33,8 @@
 #include "geo.h"
 #include "cdt_list_operations.h"
 #include "cdt_map_operations.h"
+#include "bit_operations.h"
+#include "hll_operations.h"
 
 #include <aerospike/as_double.h>
 #include <aerospike/as_integer.h>
@@ -46,6 +48,9 @@ get_operation(as_error* err, PyObject* op_dict, long* operation_ptr);
 
 static inline bool isListOp(int op);
 static inline bool isNewMapOp(int op);
+static inline bool isBitOp(int op);
+static inline bool isHllOp(int op);
+
 
 #define PY_OPERATION_KEY "op"
 
@@ -109,6 +114,12 @@ static inline bool isNewMapOp(int op);
 		return err->code;\
 	}
 
+#define CONVERT_PY_CTX_TO_AS_CTX()\
+	if (get_cdt_ctx(self, err, &ctx, py_val, &ctx_in_use,\
+			static_pool, SERIALIZER_PYTHON) != AEROSPIKE_OK) {\
+		return err->code;\
+	}
+
 #define CONVERT_RANGE_TO_AS_VAL()\
 	if (pyobject_to_astype_write(self, err, py_range, &put_range,\
 			static_pool, SERIALIZER_PYTHON) != AEROSPIKE_OK) {\
@@ -163,7 +174,7 @@ int check_type(AerospikeClient * self, PyObject * py_value, int op, as_error *er
 	if ((!PyInt_Check(py_value) && !PyLong_Check(py_value) && strcmp(py_value->ob_type->tp_name, "aerospike.null")) && (op == AS_OPERATOR_TOUCH)) {
 		as_error_update(err, AEROSPIKE_ERR_PARAM, "Unsupported operand type(s) for touch : only int or long allowed");
 		return 1;
-	} else if ((!PyInt_Check(py_value) && !PyLong_Check(py_value) && (!PyFloat_Check(py_value) || !aerospike_has_double(self->as)) &&
+	} else if ((!PyInt_Check(py_value) && !PyLong_Check(py_value) && (!PyFloat_Check(py_value) ) &&
 				strcmp(py_value->ob_type->tp_name, "aerospike.null")) && op == AS_OPERATOR_INCR) {
 		as_error_update(err, AEROSPIKE_ERR_PARAM, "Unsupported operand type(s) for +: only 'int' allowed");
 		return 1;
@@ -224,6 +235,18 @@ static inline bool isNewMapOp(int op) {
 			op == OP_MAP_GET_BY_KEY_INDEX_RANGE_REL);
 }
 
+static inline bool isBitOp(int op) {
+	int bit_start = OP_BIT_RESIZE;
+	int bit_end = OP_BIT_RSCAN;
+	return (op >= bit_start && op <= bit_end);
+}
+
+static inline bool isHllOp(int op) {
+	int hll_start = OP_HLL_ADD;
+	int hll_end = OP_HLL_SET_UNION;
+	return (op >= hll_start && op <= hll_end);
+}
+
 bool opRequiresIndex(int op) {
 	return (op == OP_LIST_INSERT               || op == OP_LIST_INSERT_ITEMS  ||
 			op == OP_LIST_POP                  || op == OP_LIST_POP_RANGE     ||
@@ -246,7 +269,8 @@ bool opRequiresValue(int op) {
 			op != OP_MAP_CLEAR           && op != OP_MAP_REMOVE_BY_KEY  &&
 			op != OP_MAP_REMOVE_BY_INDEX && op != OP_MAP_REMOVE_BY_RANK &&
 			op != OP_MAP_GET_BY_KEY      && op != OP_MAP_GET_BY_INDEX   &&
-			op != OP_MAP_GET_BY_KEY_RANGE && op != OP_MAP_GET_BY_RANK);
+			op != OP_MAP_GET_BY_KEY_RANGE && op != OP_MAP_GET_BY_RANK   &&
+			op != AS_OPERATOR_DELETE);
 }
 
 bool opRequiresRange(int op) {
@@ -282,6 +306,9 @@ as_status add_op(AerospikeClient * self, as_error * err, PyObject * py_val, as_v
 	as_val* put_val = NULL;
 	as_val* put_key = NULL;
 	as_val* put_range = NULL;
+	as_cdt_ctx ctx;
+	as_cdt_ctx* ctx_ref = NULL;
+	bool ctx_in_use = false;
 	char* bin = NULL;
 	char* val = NULL;
 	long offset = 0;
@@ -322,6 +349,16 @@ as_status add_op(AerospikeClient * self, as_error * err, PyObject * py_val, as_v
 			ops, operation, ret_type, SERIALIZER_PYTHON);
 	}
 
+    if (isBitOp(operation)) {
+		return add_new_bit_op(self, err, py_val, unicodeStrVector, static_pool,
+			ops, operation, ret_type, SERIALIZER_PYTHON);
+    }
+
+	if (isHllOp(operation)) {
+		return add_new_hll_op(self, err, py_val, unicodeStrVector, static_pool,
+			ops, operation, ret_type, SERIALIZER_PYTHON);
+	}
+
 	while (PyDict_Next(py_val, &pos, &key_op, &value)) {
 		if (!PyString_Check(key_op)) {
 			return as_error_update(err, AEROSPIKE_ERR_CLIENT, "An operation key must be a string.");
@@ -343,9 +380,11 @@ as_status add_op(AerospikeClient * self, as_error * err, PyObject * py_val, as_v
 				py_map_policy = value;
 			} else if (!strcmp(name, "return_type")) {
 				py_return_type = value;
-			}
-			else if (strcmp(name, "inverted") == 0) {
+			} else if (strcmp(name, "inverted") == 0) {
 				continue;
+			} else if (strcmp(name, "ctx") == 0) {
+				CONVERT_PY_CTX_TO_AS_CTX();
+				ctx_ref = (ctx_in_use ? &ctx : NULL);
 			} else {
 				return as_error_update(err, AEROSPIKE_ERR_PARAM,
 						"Operation can contain only op, bin, index, key, val, return_type and map_policy keys");
@@ -374,7 +413,7 @@ as_status add_op(AerospikeClient * self, as_error * err, PyObject * py_val, as_v
 				return as_error_update(err, AEROSPIKE_ERR_BIN_NAME, "A bin name should not exceed 14 characters limit");
 			}
 		}
-	} else if (operation != AS_OPERATOR_TOUCH) {
+	} else if (operation != AS_OPERATOR_TOUCH && operation != AS_OPERATOR_DELETE) {
 		return as_error_update(err, AEROSPIKE_ERR_PARAM, "Bin is not given");
 	}
 
@@ -531,6 +570,9 @@ as_status add_op(AerospikeClient * self, as_error * err, PyObject * py_val, as_v
 		case AS_OPERATOR_READ:
 			as_operations_add_read(ops, bin);
 			break;
+		case AS_OPERATOR_DELETE:
+			as_operations_add_delete(ops);
+			break;
 		case AS_OPERATOR_WRITE:
 			CONVERT_VAL_TO_AS_VAL();
 			as_operations_add_write(ops, bin, (as_bin_value *) put_val);
@@ -538,128 +580,147 @@ as_status add_op(AerospikeClient * self, as_error * err, PyObject * py_val, as_v
 
 		//------- MAP OPERATIONS ---------
 		case OP_MAP_SET_POLICY:
-			as_operations_add_map_set_policy(ops, bin, &map_policy);
+			as_operations_map_set_policy(ops, bin, ctx_ref, &map_policy);
 			break;
 		case OP_MAP_PUT:
 			CONVERT_VAL_TO_AS_VAL();
 			CONVERT_KEY_TO_AS_VAL();
-			as_operations_add_map_put(ops, bin, &map_policy, put_key, put_val);
+			as_operations_map_put(ops, bin, ctx_ref, &map_policy, put_key, put_val);
 			break;
 		case OP_MAP_PUT_ITEMS:
 			CONVERT_VAL_TO_AS_VAL();
-			as_operations_add_map_put_items(ops, bin, &map_policy, (as_map *)put_val);
+			as_operations_map_put_items(ops, bin, ctx_ref, &map_policy, (as_map *)put_val);
 			break;
 		case OP_MAP_INCREMENT:
 			CONVERT_VAL_TO_AS_VAL();
 			CONVERT_KEY_TO_AS_VAL();
-			as_operations_add_map_increment(ops, bin, &map_policy, put_key, put_val);
+			as_operations_map_increment(ops, bin, ctx_ref, &map_policy, put_key, put_val);
 			break;
 		case OP_MAP_DECREMENT:
 			CONVERT_VAL_TO_AS_VAL();
 			CONVERT_KEY_TO_AS_VAL();
-			as_operations_add_map_decrement(ops, bin, &map_policy, put_key, put_val);
+			as_operations_map_decrement(ops, bin, ctx_ref, &map_policy, put_key, put_val);
 			break;
 		case OP_MAP_SIZE:
-			as_operations_add_map_size(ops, bin);
+			as_operations_map_size(ops, bin, ctx_ref);
 			break;
 		case OP_MAP_CLEAR:
-			as_operations_add_map_clear(ops, bin);
+			as_operations_map_clear(ops, bin, ctx_ref);
 			break;
 		case OP_MAP_REMOVE_BY_KEY:
 			CONVERT_KEY_TO_AS_VAL();
-			as_operations_add_map_remove_by_key(ops, bin, put_key, return_type);
+			as_operations_map_remove_by_key(ops, bin, ctx_ref, put_key, return_type);
 			break;
 		case OP_MAP_REMOVE_BY_KEY_LIST:
 			CONVERT_VAL_TO_AS_VAL();
-			as_operations_add_map_remove_by_key_list(ops, bin, (as_list *)put_val, return_type);
+			as_operations_map_remove_by_key_list(ops, bin, ctx_ref, (as_list *)put_val, return_type);
 			break;
 		case OP_MAP_REMOVE_BY_KEY_RANGE:
 			CONVERT_VAL_TO_AS_VAL();
 			CONVERT_KEY_TO_AS_VAL();
-			as_operations_add_map_remove_by_key_range(ops, bin, put_key, put_val, return_type);
+			as_operations_map_remove_by_key_range(ops, bin, ctx_ref, put_key, put_val, return_type);
 			break;
 		case OP_MAP_REMOVE_BY_VALUE:
 			CONVERT_VAL_TO_AS_VAL();
-			as_operations_add_map_remove_by_value(ops, bin, put_val, return_type);
+			as_operations_map_remove_by_value(ops, bin, ctx_ref, put_val, return_type);
 			break;
 		case OP_MAP_REMOVE_BY_VALUE_LIST:
 			CONVERT_VAL_TO_AS_VAL();
-			as_operations_add_map_remove_by_value_list(ops, bin, (as_list *)put_val, return_type);
+			as_operations_map_remove_by_value_list(ops, bin, ctx_ref, (as_list *)put_val, return_type);
 			break;
 		case OP_MAP_REMOVE_BY_VALUE_RANGE:
 			CONVERT_VAL_TO_AS_VAL();
 			CONVERT_RANGE_TO_AS_VAL();
-			as_operations_add_map_remove_by_value_range(ops, bin, put_val, put_range, return_type);
+			as_operations_map_remove_by_value_range(ops, bin, ctx_ref, put_val, put_range, return_type);
 			break;
 		case OP_MAP_REMOVE_BY_INDEX:
-			as_operations_add_map_remove_by_index(ops, bin, index, return_type);
+			as_operations_map_remove_by_index(ops, bin, ctx_ref, index, return_type);
 			break;
 		case OP_MAP_REMOVE_BY_INDEX_RANGE:
 			if (pyobject_to_index(self, err, py_value, &offset) != AEROSPIKE_OK) {
+				if (ctx_in_use) {
+					as_cdt_ctx_destroy(&ctx);
+				}
 				return err->code;
 			}
-			as_operations_add_map_remove_by_index_range(ops, bin, index, offset, return_type);
+			as_operations_map_remove_by_index_range(ops, bin, ctx_ref, index, offset, return_type);
 			break;
 		case OP_MAP_REMOVE_BY_RANK:
-			as_operations_add_map_remove_by_rank(ops, bin, index, return_type);
+			as_operations_map_remove_by_rank(ops, bin, ctx_ref, index, return_type);
 			break;
 		case OP_MAP_REMOVE_BY_RANK_RANGE:
 			if (pyobject_to_index(self, err, py_value, &offset) != AEROSPIKE_OK) {
+				if (ctx_in_use) {
+					as_cdt_ctx_destroy(&ctx);
+				}
 				return err->code;
 			}
-			as_operations_add_map_remove_by_rank_range(ops, bin, index, offset, return_type);
+			as_operations_map_remove_by_rank_range(ops, bin, ctx_ref, index, offset, return_type);
 			break;
 		case OP_MAP_GET_BY_KEY:
 			CONVERT_KEY_TO_AS_VAL();
-			as_operations_add_map_get_by_key(ops, bin, put_key, return_type);
+			as_operations_map_get_by_key(ops, bin, ctx_ref, put_key, return_type);
 			break;
 		case OP_MAP_GET_BY_KEY_RANGE:
 			CONVERT_RANGE_TO_AS_VAL();
 			CONVERT_KEY_TO_AS_VAL();
-			as_operations_add_map_get_by_key_range(ops, bin, put_key, put_range, return_type);
+			as_operations_map_get_by_key_range(ops, bin, ctx_ref, put_key, put_range, return_type);
 			break;
 		case OP_MAP_GET_BY_KEY_LIST:
 			CONVERT_VAL_TO_AS_VAL();
-			as_operations_add_map_get_by_key_list(ops, bin, (as_list *)put_val, return_type);
+			as_operations_map_get_by_key_list(ops, bin, ctx_ref, (as_list *)put_val, return_type);
 			break;
 		case OP_MAP_GET_BY_VALUE:
 			CONVERT_VAL_TO_AS_VAL();
-			as_operations_add_map_get_by_value(ops, bin, put_val, return_type);
+			as_operations_map_get_by_value(ops, bin, ctx_ref, put_val, return_type);
 			break;
 		case OP_MAP_GET_BY_VALUE_RANGE:
 			CONVERT_VAL_TO_AS_VAL();
 			CONVERT_RANGE_TO_AS_VAL();
-			as_operations_add_map_get_by_value_range(ops, bin, put_val, put_range, return_type);
+			as_operations_map_get_by_value_range(ops, bin, ctx_ref, put_val, put_range, return_type);
 			break;
 		case OP_MAP_GET_BY_VALUE_LIST:
 			CONVERT_VAL_TO_AS_VAL();
-			as_operations_add_map_get_by_value_list(ops, bin, (as_list *)put_val, return_type);
+			as_operations_map_get_by_value_list(ops, bin, ctx_ref, (as_list *)put_val, return_type);
 			break;
 		case OP_MAP_GET_BY_INDEX:
-			as_operations_add_map_get_by_index(ops, bin, index, return_type);
+			as_operations_map_get_by_index(ops, bin, ctx_ref, index, return_type);
 			break;
 		case OP_MAP_GET_BY_INDEX_RANGE:
 			if (pyobject_to_index(self, err, py_value, &offset) != AEROSPIKE_OK) {
+				if (ctx_in_use) {
+					as_cdt_ctx_destroy(&ctx);
+				}
 				return err->code;
 			}
-			as_operations_add_map_get_by_index_range(ops, bin, index, offset, return_type);
+			as_operations_map_get_by_index_range(ops, bin, ctx_ref, index, offset, return_type);
 			break;
 		case OP_MAP_GET_BY_RANK:
-			as_operations_add_map_get_by_rank(ops, bin, index, return_type);
+			as_operations_map_get_by_rank(ops, bin, ctx_ref, index, return_type);
 			break;
 		case OP_MAP_GET_BY_RANK_RANGE:
 			if (pyobject_to_index(self, err, py_value, &offset) != AEROSPIKE_OK) {
+				if (ctx_in_use) {
+					as_cdt_ctx_destroy(&ctx);
+				}
 				return err->code;
 			}
-			as_operations_add_map_get_by_rank_range(ops, bin, index, offset, return_type);
+			as_operations_map_get_by_rank_range(ops, bin, ctx_ref, index, offset, return_type);
 			break;
 
 		default:
 			if (self->strict_types) {
+				if (ctx_in_use) {
+					as_cdt_ctx_destroy(&ctx);
+				}
 				return as_error_update(err, AEROSPIKE_ERR_PARAM, "Invalid operation given");
 			}
 	}
 
+	if (ctx_in_use) {
+		as_cdt_ctx_destroy(&ctx);
+	}
+
 	return err->code;
 }
 
@@ -686,11 +747,16 @@ PyObject *  AerospikeClient_Operate_Invoke(
 	int i = 0;
 	long operation;
 	long return_type = -1;
+	bool operation_succeeded = false;
 	PyObject * py_rec = NULL;
 	as_record * rec = NULL;
 	as_policy_operate operate_policy;
 	as_policy_operate *operate_policy_p = NULL;
 
+	// For predexp conversion.
+	as_predexp_list predexp_list;
+	as_predexp_list* predexp_list_p = NULL;
+
 	as_vector * unicodeStrVector = as_vector_create(sizeof(char *), 128);
 
 	as_operations ops;
@@ -699,14 +765,13 @@ PyObject *  AerospikeClient_Operate_Invoke(
 
 	if (py_policy) {
 		if(pyobject_to_policy_operate(err, py_policy, &operate_policy, &operate_policy_p,
-				&self->as->config.policies.operate) != AEROSPIKE_OK) {
+				&self->as->config.policies.operate, &predexp_list, &predexp_list_p) != AEROSPIKE_OK) {
 			goto CLEANUP;
 		}
 	}
 
 	as_static_pool static_pool;
 	memset(&static_pool, 0, sizeof(static_pool));
-
 	CHECK_CONNECTED(err);
 
 	if (py_meta) {
@@ -729,9 +794,6 @@ PyObject *  AerospikeClient_Operate_Invoke(
 		goto CLEANUP;
 	}
 
-	// Initialize record
-	as_record_init(rec, 0);
-
 	Py_BEGIN_ALLOW_THREADS
 	aerospike_key_operate(self->as, err, operate_policy_p, key, &ops, &rec);
 	Py_END_ALLOW_THREADS
@@ -740,6 +802,9 @@ PyObject *  AerospikeClient_Operate_Invoke(
 		as_error_update(err, err->code, NULL);
 		goto CLEANUP;
 	}
+	/* The op succeeded; it's now safe to free the record */
+	operation_succeeded = true;
+
 	if (rec) {
 		record_to_pyobject(self, err, rec, key, &py_rec);
 	}
@@ -749,9 +814,13 @@ CLEANUP:
 		free(as_vector_get_ptr(unicodeStrVector, i));
 	}
 
+	if (predexp_list_p) {
+		as_predexp_list_destroy(&predexp_list);
+	}
+	
 	as_vector_destroy(unicodeStrVector);
 
-	if (rec) {
+	if (rec && operation_succeeded) {
 		as_record_destroy(rec);
 	}
 	if (key->valuep) {
@@ -843,6 +912,8 @@ static PyObject *  AerospikeClient_OperateOrdered_Invoke(
 {
 	long operation;
 	long return_type = -1;
+	bool operation_succeeded = false;
+
 	PyObject * py_rec = NULL;
 	as_record * rec = NULL;
 	as_policy_operate operate_policy;
@@ -857,6 +928,10 @@ static PyObject *  AerospikeClient_OperateOrdered_Invoke(
 	Py_ssize_t ops_list_size = PyList_Size(py_list);
 	as_operations_inita(&ops, ops_list_size);
 
+	// For predexp conversion.
+	as_predexp_list predexp_list;
+	as_predexp_list* predexp_list_p = NULL;
+
 	/* These are the values which will be returned in a 3 element list */
 	PyObject* py_return_key = NULL;
 	PyObject* py_return_meta = NULL;
@@ -866,7 +941,7 @@ static PyObject *  AerospikeClient_OperateOrdered_Invoke(
 
 	if (py_policy) {
 		if (pyobject_to_policy_operate(err, py_policy, &operate_policy,
-				&operate_policy_p, &self->as->config.policies.operate) != AEROSPIKE_OK) {
+				&operate_policy_p, &self->as->config.policies.operate, &predexp_list, &predexp_list_p) != AEROSPIKE_OK) {
 			goto CLEANUP;
 		}
 	}
@@ -898,9 +973,6 @@ static PyObject *  AerospikeClient_OperateOrdered_Invoke(
 		goto CLEANUP;
 	}
 
-	// Initialize record
-	as_record_init(rec, 0);
-
 	Py_BEGIN_ALLOW_THREADS
 	aerospike_key_operate(self->as, err, operate_policy_p, key, &ops, &rec);
 	Py_END_ALLOW_THREADS
@@ -910,6 +982,7 @@ static PyObject *  AerospikeClient_OperateOrdered_Invoke(
 		goto CLEANUP;
 	}
 
+	operation_succeeded = true;
 	if (rec) {
 		/* Build the return tuple: (key, meta, bins) */
 		key_to_pyobject(err, key, &py_return_key);
@@ -953,7 +1026,11 @@ CLEANUP:
 
 	as_vector_destroy(unicodeStrVector);
 
-	if (rec) {
+	if (predexp_list_p) {
+		as_predexp_list_destroy(&predexp_list);
+	}
+
+	if (rec && operation_succeeded) {
 		as_record_destroy(rec);
 	}
 	if (key->valuep) {
