@@ -55,7 +55,7 @@
 #define POLICY_KEY_META_BIN()\
 	if (py_policy) {\
 		if (pyobject_to_policy_operate(&err, py_policy, &operate_policy, &operate_policy_p,\
-				&self->as->config.policies.operate) != AEROSPIKE_OK) {\
+				&self->as->config.policies.operate, NULL, NULL) != AEROSPIKE_OK) {\
 			goto CLEANUP;\
 		}\
 	}\
@@ -93,7 +93,8 @@
 #define DO_OPERATION()\
 	Py_BEGIN_ALLOW_THREADS\
 	aerospike_key_operate(self->as, &err, operate_policy_p, &key, &ops, &rec);\
-	Py_END_ALLOW_THREADS
+	Py_END_ALLOW_THREADS\
+    if (err.code != AEROSPIKE_OK) { goto CLEANUP;}
 
 #define SETUP_RETURN_VAL()\
 	if (rec && rec->bins.size) {\
@@ -102,7 +103,9 @@
 		} else {\
 			val_to_pyobject(self, &err, (as_val*) (rec->bins.entries[0].valuep), &py_result);\
 		}\
-	}
+	} else {\
+        as_error_update(&err, AEROSPIKE_ERR_CLIENT, "Unexpected empty return");\
+    }
 
 #define CLEANUP_AND_EXCEPTION_ON_ERROR(__err)\
 	as_operations_destroy(&ops);\
@@ -387,7 +390,7 @@ PyObject * AerospikeClient_MapSize(AerospikeClient * self, PyObject * args, PyOb
 	as_operations_add_map_size(&ops, bin);
 	DO_OPERATION();
 
-	if (rec && as_val_type(rec->bins.entries[0].valuep) != AS_NIL) {
+	if (rec && rec->bins.entries &&  rec->bins.size > 0 && as_val_type(rec->bins.entries[0].valuep) != AS_NIL) {
 		size = rec->bins.entries[0].valuep->integer.value;
 	}
 
