@@ -51,6 +51,10 @@ PyObject * AerospikeClient_Remove_Invoke(
 	as_policy_remove * remove_policy_p = NULL;
 	as_key key;
 
+	// For converting predexp.
+	as_predexp_list predexp_list;
+	as_predexp_list* predexp_list_p = NULL;
+
 	// Initialisation flags
 	bool key_initialised = false;
 
@@ -78,7 +82,7 @@ PyObject * AerospikeClient_Remove_Invoke(
 	// Convert python policy object to as_policy_exists
 	if (py_policy) {
 		pyobject_to_policy_remove(&err, py_policy, &remove_policy, &remove_policy_p,
-				&self->as->config.policies.remove);
+				&self->as->config.policies.remove, &predexp_list, &predexp_list_p);
 		if (err.code != AEROSPIKE_OK) {
 			goto CLEANUP;
 		} else {
@@ -112,6 +116,10 @@ PyObject * AerospikeClient_Remove_Invoke(
 
 CLEANUP:
 
+	if (predexp_list_p) {
+		as_predexp_list_destroy(&predexp_list);
+	}
+
 	if (key_initialised == true) {
 		// Destroy the key if it is initialised successfully.
 		as_key_destroy(&key);
