@@ -57,6 +57,10 @@ PyObject * AerospikeClient_RemoveBin_Invoke(
 	int count = 0;
 	PyObject * py_ustr = NULL;
 
+	// For converting predexp.
+	as_predexp_list predexp_list;
+	as_predexp_list* predexp_list_p = NULL;
+
 	// Get the bin list size;
 	Py_ssize_t size = PyList_Size(py_binList);
 	// Initialize record
@@ -71,7 +75,7 @@ PyObject * AerospikeClient_RemoveBin_Invoke(
 
 	// Convert python policy object to as_policy_write
 	pyobject_to_policy_write(err, py_policy, &write_policy, &write_policy_p,
-			&self->as->config.policies.write);
+			&self->as->config.policies.write, &predexp_list, &predexp_list_p);
 	if (err->code != AEROSPIKE_OK) {
 		as_error_update(err, AEROSPIKE_ERR_CLIENT, "Incorrect policy");
 		goto CLEANUP;
@@ -155,6 +159,10 @@ CLEANUP:
 
 	as_record_destroy(&rec);
 
+	if (predexp_list_p) {
+		as_predexp_list_destroy(&predexp_list);
+	}
+
 	if (key_initialized) {
 		as_key_destroy(&key);
 	}
