@@ -32,7 +32,11 @@
 #include "tls_config.h"
 #include "policy_config.h"
 
-enum {INIT_NO_CONFIG_ERR = 1, INIT_CONFIG_TYPE_ERR, INIT_LUA_USER_ERR,
+
+static int set_rack_aware_config(as_config* conf, PyObject* config_dict);
+static int set_use_services_alternate(as_config* conf, PyObject* config_dict);
+
+enum {INIT_SUCCESS, INIT_NO_CONFIG_ERR, INIT_CONFIG_TYPE_ERR, INIT_LUA_USER_ERR,
 	  INIT_LUA_SYS_ERR,  INIT_HOST_TYPE_ERR, INIT_EMPTY_HOSTS_ERR,
 	  INIT_INVALID_ADRR_ERR, INIT_SERIALIZE_ERR, INIT_DESERIALIZE_ERR,
 	  INIT_COMPRESSION_ERR, INIT_POLICY_PARAM_ERR};
@@ -368,7 +372,7 @@ Return a `aerospike.Scan` object to be used for executing scans over a specified
 (which can be omitted or None) in a namespace. A scan with a None set returns all the records in the namespace.");
 
 PyDoc_STRVAR(scan_apply_doc,
-"scan_apply(ns, set, module, function[, args[, policy[, options]]]) -> int\n\
+"scan_apply(ns, set, module, function[, args[, policy[, options,[ block]]]]) -> int\n\
 \n\
 Initiate a background scan and apply a record UDF to each record matched by the scan.");
 
@@ -874,23 +878,11 @@ static int AerospikeClient_Type_Init(AerospikeClient * self, PyObject * args, Py
 	as_config config;
 	as_config_init(&config);
 
-	bool lua_system_path = false;
 	bool lua_user_path = false;
 
 	PyObject * py_lua = PyDict_GetItemString(py_config, "lua");
 	if (py_lua && PyDict_Check(py_lua)) {
 
-		PyObject * py_lua_system_path = PyDict_GetItemString(py_lua, "system_path");
-		if (py_lua_system_path && PyString_Check(py_lua_system_path)) {
-			lua_system_path = true;
-			if (strnlen(PyString_AsString(py_lua_system_path), AS_CONFIG_PATH_MAX_SIZE) >
-			    AS_CONFIG_PATH_MAX_LEN) {
-					error_code = INIT_LUA_SYS_ERR;
-					goto CONSTRUCTOR_ERROR;
-
-			}
-			strcpy(config.lua.system_path, PyString_AsString(py_lua_system_path));
-		}
 
 		PyObject * py_lua_user_path = PyDict_GetItemString(py_lua, "user_path");
 		if (py_lua_user_path && PyString_Check(py_lua_user_path)) {
@@ -904,19 +896,6 @@ static int AerospikeClient_Type_Init(AerospikeClient * self, PyObject * args, Py
 		}
 	}
 
-	if (!lua_system_path) {
-		char system_path[AS_CONFIG_PATH_MAX_LEN];
-		strcpy(system_path, "/usr/local/aerospike/lua");
-
-		struct stat info;
-		if (stat(system_path, &info) == 0 && (info.st_mode & S_IFDIR) && (access(system_path, R_OK)) == 0) {
-			strcpy(config.lua.system_path, system_path);
-		}
-		else {
-			config.lua.system_path[0] = '\0';
-		}
-	}
-
 	if (!lua_user_path) {
 		strcpy(config.lua.user_path, ".");
 	} else {
@@ -1072,7 +1051,6 @@ static int AerospikeClient_Type_Init(AerospikeClient * self, PyObject * args, Py
 
 	as_policies_init(&config.policies);
 	//Set default value of use_batch_direct
-	config.policies.batch.use_batch_direct = false;
 
 	PyObject * py_policies = PyDict_GetItemString(py_config, "policies");
 	if (py_policies && PyDict_Check(py_policies)) {
@@ -1161,12 +1139,20 @@ static int AerospikeClient_Type_Init(AerospikeClient * self, PyObject * args, Py
 			config.policies.remove.replica = long_replica;
 		}
 
-		PyObject * py_consistency_level = PyDict_GetItemString(py_policies, "consistency_level");
-		if (py_consistency_level && PyInt_Check(py_consistency_level)) {
-			long long_consistency_level = PyInt_AsLong(py_consistency_level);
-			config.policies.read.consistency_level = long_consistency_level;
-			config.policies.operate.consistency_level = long_consistency_level;
-			config.policies.batch.consistency_level = long_consistency_level;
+		PyObject * py_ap_read_mode = PyDict_GetItemString(py_policies, "read_mode_ap");
+		if (py_ap_read_mode && PyInt_Check(py_ap_read_mode)) {
+			as_policy_read_mode_ap ap_read_mode = (as_policy_read_mode_ap)PyInt_AsLong(py_ap_read_mode);
+			config.policies.read.read_mode_ap = ap_read_mode;
+			config.policies.operate.read_mode_ap = ap_read_mode;
+			config.policies.batch.read_mode_ap = ap_read_mode;
+		}
+
+		PyObject * py_sc_read_mode = PyDict_GetItemString(py_policies, "read_mode_sc");
+		if (py_sc_read_mode && PyInt_Check(py_sc_read_mode)) {
+			as_policy_read_mode_sc sc_read_mode = (as_policy_read_mode_sc)PyInt_AsLong(py_sc_read_mode);
+			config.policies.read.read_mode_sc = sc_read_mode;
+			config.policies.operate.read_mode_sc = sc_read_mode;
+			config.policies.batch.read_mode_sc = sc_read_mode;
 		}
 
 		PyObject * py_commit_level = PyDict_GetItemString(py_policies, "commit_level");
@@ -1192,13 +1178,6 @@ static int AerospikeClient_Type_Init(AerospikeClient * self, PyObject * args, Py
 			config.thread_pool_size = PyInt_AsLong(py_thread_pool_size);
 		}
 
-		// This does not match documentation (wrong name and location in dict),
-		//  but leave it for now for customers who may be using it
-		PyObject * py_use_batch_direct = PyDict_GetItemString(py_policies, "use_batch_direct");
-		if (py_use_batch_direct && PyBool_Check(py_use_batch_direct)) {
-			config.policies.batch.use_batch_direct = PyInt_AsLong(py_use_batch_direct);
-		}
-
 		/*
 		 * Generation policy is removed from constructor.
 		 */
@@ -1247,11 +1226,6 @@ static int AerospikeClient_Type_Init(AerospikeClient * self, PyObject * args, Py
 		config.max_conns_per_node = PyInt_AsLong(py_max_conns);
 	}
 
-	// batch_direct
-	PyObject * py_batch_direct = PyDict_GetItemString(py_config, "batch_direct");
-	if (py_batch_direct && PyBool_Check(py_batch_direct)) {
-		config.policies.batch.use_batch_direct = PyInt_AsLong(py_batch_direct);
-	}
 
 	//conn_timeout_ms
 	PyObject * py_connect_timeout = PyDict_GetItemString(py_config, "connect_timeout");
@@ -1296,6 +1270,15 @@ static int AerospikeClient_Type_Init(AerospikeClient * self, PyObject * args, Py
  		}
 	}
 
+    if (set_rack_aware_config(&config, py_config) != INIT_SUCCESS) {
+        error_code = INIT_POLICY_PARAM_ERR;
+        goto CONSTRUCTOR_ERROR;
+    }
+    if (set_use_services_alternate(&config, py_config) != INIT_SUCCESS) {
+        error_code = INIT_POLICY_PARAM_ERR;
+        goto CONSTRUCTOR_ERROR;
+    }
+
 	PyObject* py_max_socket_idle = NULL;
 	py_max_socket_idle = PyDict_GetItemString(py_config, "max_socket_idle");
 	if (py_max_socket_idle && PyInt_Check(py_max_socket_idle)) {
@@ -1375,6 +1358,52 @@ CONSTRUCTOR_ERROR:
 
 }
 
+static int set_rack_aware_config(as_config*conf, PyObject* config_dict) {
+    PyObject* py_config_value;
+    long rack_id;
+    py_config_value = PyDict_GetItemString(config_dict, "rack_aware");
+    if (py_config_value) {
+        if (PyBool_Check(py_config_value)) {
+            conf->rack_aware = PyObject_IsTrue(py_config_value);
+        } else {
+            return INIT_POLICY_PARAM_ERR;  // A non boolean was passed in as the value of rack_aware
+        }
+    }
+
+    py_config_value = PyDict_GetItemString(config_dict, "rack_id");
+    if (py_config_value) {
+        if (PyLong_Check(py_config_value)) {
+            rack_id = PyLong_AsLong(py_config_value);
+        } else if (PyInt_Check(py_config_value)) {
+            rack_id = PyInt_AsLong(py_config_value);
+        } else {
+            return INIT_POLICY_PARAM_ERR;  // A non integer passed in.
+        }
+        if (rack_id == -1 && PyErr_Occurred()) {
+            return INIT_POLICY_PARAM_ERR; // We had overflow.
+        }
+
+        if (rack_id > INT_MAX || rack_id < INT_MIN) {
+            return INIT_POLICY_PARAM_ERR; // Magnitude too great for an integer in C.
+        }
+        conf->rack_id = (int)rack_id;
+    }
+    return INIT_SUCCESS;
+}
+
+static int set_use_services_alternate(as_config* conf, PyObject* config_dict) {
+    PyObject* py_config_value;
+    py_config_value = PyDict_GetItemString(config_dict, "use_services_alternate");
+    if (py_config_value) {
+        if (PyBool_Check(py_config_value)) {
+            conf->use_services_alternate = PyObject_IsTrue(py_config_value);
+        } else {
+            return INIT_POLICY_PARAM_ERR;  // A non boolean was passed in as the value of use_services_alternate
+        }
+    }
+    return INIT_SUCCESS;
+}
+
 static void AerospikeClient_Type_Dealloc(PyObject * self)
 {
 
