@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2017 Aerospike, Inc.
+ * Copyright 2013-2019 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -35,12 +35,14 @@
 #include <aerospike/as_bytes.h>
 #include <aerospike/as_double.h>
 #include <aerospike/as_record_iterator.h>
+#include <aerospike/as_msgpack_ext.h>
 
 #include "conversions.h"
 #include "geo.h"
 #include "policy.h"
 #include "serializer.h"
 #include "exceptions.h"
+#include "cdt_types.h"
 
 #define PY_KEYT_NAMESPACE 0
 #define PY_KEYT_SET 1
@@ -53,6 +55,10 @@
 #define PY_EXCEPTION_LINE 3
 #define AS_PY_EXCEPTION_IN_DOUBT 4
 
+#define CTX_KEY "ctx"
+
+bool requires_int(uint64_t op);
+
 as_status as_udf_file_to_pyobject( as_error *err, as_udf_file * entry, PyObject ** py_file )
 {
 	as_error_reset(err);
@@ -431,6 +437,10 @@ as_status pyobject_to_val(AerospikeClient * self, as_error * err, PyObject * py_
 	} else if (PyString_Check(py_obj)) {
 		char * s = PyString_AsString(py_obj);
 		*val = (as_val *) as_string_new(s, false);
+	 } else if (PyBytes_Check(py_obj)) {
+	 	uint8_t * b = (uint8_t *) PyBytes_AsString(py_obj);
+	 	uint32_t b_len  = (uint32_t)  PyBytes_Size(py_obj);
+	 	*val = (as_val *) as_bytes_new_wrap(b, b_len, false);
 	} else if (!strcmp(py_obj->ob_type->tp_name, "aerospike.Geospatial")) {
 		PyObject *py_parameter = PyString_FromString("geo_data");
 		PyObject* py_data = PyObject_GenericGetAttr(py_obj, py_parameter);
@@ -474,9 +484,13 @@ as_status pyobject_to_val(AerospikeClient * self, as_error * err, PyObject * py_
 	} else if (Py_None == py_obj) {
 		*val = as_val_reserve(&as_nil);
 	} else if (!strcmp(py_obj->ob_type->tp_name, "aerospike.null")) {
-		*val = (as_val *) &as_nil;
+		*val = (as_val *) as_val_reserve(&as_nil);
+	} else if (AS_Matches_Classname(py_obj, AS_CDT_WILDCARD_NAME)) {
+		*val = (as_val *) as_val_reserve(&as_cmp_wildcard);
+	} else if (AS_Matches_Classname(py_obj, AS_CDT_INFINITE_NAME)) {
+		*val = (as_val *) as_val_reserve(&as_cmp_inf);
 	} else {
-		if (aerospike_has_double(self->as) && PyFloat_Check(py_obj)) {
+		if (PyFloat_Check(py_obj)) {
 			double d = PyFloat_AsDouble(py_obj);
 			*val = (as_val *) as_double_new(d);
 		} else {
@@ -645,7 +659,7 @@ as_status pyobject_to_record(AerospikeClient * self, as_error * err, PyObject *
 			} else if (!strcmp(value->ob_type->tp_name, "aerospike.null")) {
 				ret_val = as_record_set_nil(rec, name);
 			} else {
-				if (aerospike_has_double(self->as) && PyFloat_Check(value)) {
+				if (PyFloat_Check(value)) {
 					double val = PyFloat_AsDouble(value);
 					ret_val = as_record_set_double(rec, name, val);
 				} else {
@@ -800,10 +814,14 @@ as_status pyobject_to_astype_write(AerospikeClient * self, as_error * err, PyObj
 		if (err->code == AEROSPIKE_OK) {
 			*val = (as_val *) map;
 		}
-	} else if (!strcmp(py_value->ob_type->tp_name, "aerospike.null")) {
+	} else if (AS_Matches_Classname(py_value, "aerospike.null")) {
 		*val = (as_val *) &as_nil;
-	} else {
-		if (aerospike_has_double(self->as) && PyFloat_Check(py_value)) {
+	} else if (AS_Matches_Classname(py_value, AS_CDT_WILDCARD_NAME)) {
+		*val = (as_val *) as_val_reserve(&as_cmp_wildcard);
+	} else if (AS_Matches_Classname(py_value, AS_CDT_INFINITE_NAME)) {
+		*val = (as_val *) as_val_reserve(&as_cmp_inf);
+	}else {
+		if (PyFloat_Check(py_value)) {
 			double d = PyFloat_AsDouble(py_value);
 			*val = (as_val *) as_double_new(d);
 		} else {
@@ -988,11 +1006,19 @@ as_status do_val_to_pyobject(AerospikeClient * self, as_error * err, const as_va
 		case AS_INTEGER: {
 				as_integer * i = as_integer_fromval(val);
 				*py_val = PyInt_FromLong((long) as_integer_get(i));
+				if (! *py_val) {
+					as_error_update(err, AEROSPIKE_ERR_CLIENT, "Failed to create integer or long.");
+				}
 				break;
 			}
 		case AS_DOUBLE: {
 				as_double * d = as_double_fromval(val);
 				*py_val = PyFloat_FromDouble(as_double_get(d));
+
+				if (! *py_val) {
+					as_error_update(err, AEROSPIKE_ERR_CLIENT, "Failed to create float.");
+				}
+
 				break;
 			}
 		case AS_STRING: {
@@ -1000,7 +1026,7 @@ as_status do_val_to_pyobject(AerospikeClient * self, as_error * err, const as_va
 				char * str = as_string_get(s);
 				if (str) {
 					*py_val = PyString_FromString( str );
-					if (!py_val){
+					if (! *py_val){
 						size_t sz = strlen(str);
 						*py_val = PyUnicode_DecodeUTF8(str, sz, NULL);
 					}
@@ -1070,8 +1096,11 @@ as_status do_val_to_pyobject(AerospikeClient * self, as_error * err, const as_va
 			char * locstr = as_geojson_get(gp);
 			PyObject *py_locstr = PyString_FromString(locstr);
 			PyObject *py_loads = AerospikeGeospatial_DoLoads(py_locstr, err);
-			*py_val = AerospikeGeospatial_New(err, py_loads);
 			Py_DECREF(py_locstr);
+			if (err->code != AEROSPIKE_OK) {
+				break;
+			}
+			*py_val = AerospikeGeospatial_New(err, py_loads);
 			if (py_loads) {
 				Py_DECREF(py_loads);
 			}
@@ -1095,9 +1124,7 @@ as_status val_to_pyobject_cnvt_list_to_map(AerospikeClient * self, as_error * er
 }
 
 as_status as_list_of_map_to_py_tuple_list(AerospikeClient * self, as_error * err, const as_list * list, PyObject ** py_list) {
-	PyObject * py_key;
-	PyObject * py_value;
-	PyObject * py_tuple;
+	PyObject * py_tuple = NULL;
 
 	int size = as_list_size((as_list *)list);
 
@@ -1106,18 +1133,38 @@ as_status as_list_of_map_to_py_tuple_list(AerospikeClient * self, as_error * err
 	}
 
 	*py_list = PyList_New(0);
+	if (! *py_list) {
+		return as_error_update(err, AEROSPIKE_ERR_CLIENT, "Failed to allocate memory for list.");
+	}
 
 	for (int i=0; i<size; i+=2) {
 		as_val * key = as_list_get(list, i);
 		as_val * value = as_list_get(list, i+1);
 
+		if (!key || !value) {
+			as_error_update(err, AEROSPIKE_ERR_CLIENT, "Null object found in returned list");
+			goto CLEANUP;
+		}
+
+		PyObject * py_key = NULL;
+		PyObject * py_value = NULL;
+
 		if (val_to_pyobject(self, err, key, &py_key) != AEROSPIKE_OK) {
 			goto CLEANUP;
 		}
 		if (val_to_pyobject(self, err, value, &py_value) != AEROSPIKE_OK) {
+			Py_XDECREF(py_key);
 			goto CLEANUP;
 		}
 		py_tuple = PyTuple_New(2);
+
+		if (!py_tuple) {
+			as_error_update(err, AEROSPIKE_ERR_CLIENT, "Failed to allocate memory for tuple");
+			Py_XDECREF(py_key);
+			Py_XDECREF(py_value);
+			goto CLEANUP;
+		}
+
 		PyTuple_SetItem(py_tuple, 0, py_key);
 		PyTuple_SetItem(py_tuple, 1, py_value);
 
@@ -1160,6 +1207,10 @@ as_status list_to_pyobject(AerospikeClient * self, as_error * err, const as_list
 {
 	*py_list = PyList_New(as_list_size((as_list *) list));
 
+	if (! *py_list ) {
+		return as_error_update(err, AEROSPIKE_ERR_CLIENT, "Failed to allocate memory for list");
+	}
+
 	conversion_data convd = {
 		.err = err,
 		.count = 0,
@@ -1170,7 +1221,7 @@ as_status list_to_pyobject(AerospikeClient * self, as_error * err, const as_list
 	as_list_foreach(list, list_to_pyobject_each, &convd);
 
 	if (err->code != AEROSPIKE_OK) {
-		Py_DECREF(*py_list);
+		Py_CLEAR(*py_list);
 		return err->code;
 	}
 
@@ -1179,12 +1230,15 @@ as_status list_to_pyobject(AerospikeClient * self, as_error * err, const as_list
 
 static bool map_to_pyobject_each(const as_val * key, const as_val * val, void * udata)
 {
-	if (!key || !val) {
-		return false;
-	}
 
 	conversion_data * convd = (conversion_data *) udata;
 	as_error * err = convd->err;
+
+	if (!key || !val) {
+		as_error_update(err, AEROSPIKE_ERR_CLIENT, "Received null key or value");
+		return false;
+	}
+
 	PyObject * py_dict = (PyObject *) convd->udata;
 
 	PyObject * py_key = NULL;
@@ -1227,6 +1281,10 @@ as_status map_to_pyobject(AerospikeClient * self, as_error * err, const as_map *
 {
 	*py_map = PyDict_New();
 
+	if (! *py_map) {
+		return as_error_update(err, AEROSPIKE_ERR_CLIENT, "Failed to allocate memory for dictionary.");
+	}
+
 	conversion_data convd = {
 		.err = err,
 		.count = 0,
@@ -1258,7 +1316,6 @@ as_status do_record_to_pyobject(AerospikeClient * self, as_error * err, const as
 	PyObject * py_rec_meta = NULL;
 	PyObject * py_rec_bins = NULL;
 
-
 	if (key_to_pyobject(err, key ? key : &rec->key, &py_rec_key) != AEROSPIKE_OK) {
 		return err->code;
 	}
@@ -1615,17 +1672,8 @@ void initialize_bin_for_strictypes(AerospikeClient *self, as_error *err, PyObjec
 		Py_XDECREF(py_ustr1);
 	} else if (PyFloat_Check(py_value)) {
 		int64_t val = PyFloat_AsDouble(py_value);
-		if (aerospike_has_double(self->as)) {
-			as_double_init((as_double *) &binop_bin->value, val);
-			binop_bin->valuep = &binop_bin->value;
-		} else {
-			as_bytes *bytes;
-			GET_BYTES_POOL(bytes, static_pool, err);
-			serialize_based_on_serializer_policy(self, SERIALIZER_PYTHON,
-					&bytes, py_value, err);
-			((as_val *) &binop_bin->value)->type = AS_UNKNOWN;
-			binop_bin->valuep = (as_bin_value *) bytes;
-		}
+		as_double_init((as_double *) &binop_bin->value, val);
+		binop_bin->valuep = &binop_bin->value;
 	} else if (PyList_Check(py_value)) {
 		as_list * list = NULL;
 		pyobject_to_list(self, err, py_value, &list, static_pool, SERIALIZER_PYTHON);
@@ -1905,4 +1953,110 @@ string_and_pyuni_from_pystring(PyObject* py_string, PyObject** pyuni_r, char** c
 		return AEROSPIKE_OK;
 	}
 	return as_error_update(err, AEROSPIKE_ERR_PARAM, "String value required");
+}
+
+// This function converts a list of cdt_ctx from aerospike_helpers.ctx to
+// an as_cdt_ctx object for use with the c-client. the cdt_ctx parameter should be an uninitialized as_cdt_ctx
+// object. This function will initilaise it, and free it IF an error occurs, otherwise, the caller must destroy
+// the as_cdt_ctx when it is done.
+as_status get_cdt_ctx(AerospikeClient* self, as_error* err, as_cdt_ctx* cdt_ctx, 
+    PyObject* op_dict, bool* ctx_in_use, as_static_pool* static_pool, int serializer_type)
+{
+    PyObject* py_ctx = PyDict_GetItemString(op_dict, CTX_KEY);
+    int int_val = 0;
+    as_val* val = NULL;
+
+    if (!py_ctx) {
+        return AEROSPIKE_OK;
+    }
+
+    if (PyList_Check(py_ctx)) {
+        Py_ssize_t py_list_size = PyList_Size(py_ctx);
+        as_cdt_ctx_init(cdt_ctx, (int)py_list_size);
+
+        for (int i = 0; i < py_list_size; i++) {
+            PyObject* py_val = PyList_GetItem(py_ctx, (Py_ssize_t)i);
+
+            PyObject* id_temp = PyObject_GetAttrString(py_val, "id");
+            if (PyErr_Occurred()) {
+				as_cdt_ctx_destroy(cdt_ctx);
+                return as_error_update(err, AEROSPIKE_ERR_PARAM, "Failed to convert %s", CTX_KEY);
+            }
+
+            PyObject* value_temp = PyObject_GetAttrString(py_val, "value");
+            if (PyErr_Occurred()) {
+				as_cdt_ctx_destroy(cdt_ctx);
+                return as_error_update(err, AEROSPIKE_ERR_PARAM, "Failed to convert %s", CTX_KEY);
+            }
+
+            uint64_t item_type = PyLong_AsUnsignedLong(id_temp);
+            if (PyErr_Occurred()) {
+				as_cdt_ctx_destroy(cdt_ctx);
+                return as_error_update(err, AEROSPIKE_ERR_PARAM, "Failed to convert %s", CTX_KEY);
+            }
+
+            // add an as_cdt_ctx with value to cdt_ctx
+            if (requires_int(item_type)) {
+                int_val = PyLong_AsLong(value_temp);
+                if (PyErr_Occurred()) {
+					as_cdt_ctx_destroy(cdt_ctx);
+                    return as_error_update(err, AEROSPIKE_ERR_PARAM, "Failed to convert %s", CTX_KEY);
+                }
+                switch(item_type) {
+                    case AS_CDT_CTX_LIST_INDEX:
+                        as_cdt_ctx_add_list_index(cdt_ctx, int_val);
+                        break;
+                    case AS_CDT_CTX_LIST_RANK:
+                        as_cdt_ctx_add_list_rank(cdt_ctx, int_val);
+                        break;
+                    case AS_CDT_CTX_MAP_INDEX:
+                        as_cdt_ctx_add_map_index(cdt_ctx, int_val);
+                        break;
+                    case AS_CDT_CTX_MAP_RANK:
+                        as_cdt_ctx_add_map_rank(cdt_ctx, int_val);
+                        break;
+                    default:
+						as_cdt_ctx_destroy(cdt_ctx);
+                        return as_error_update(err, AEROSPIKE_ERR_PARAM, "Failed to convert, unknown ctx operation %s", CTX_KEY);
+                }
+            } else {
+                if (pyobject_to_val(self, err, value_temp, &val, static_pool, serializer_type) != AEROSPIKE_OK) {
+                    return as_error_update(err, AEROSPIKE_ERR_PARAM, "Failed to convert %s", CTX_KEY);
+                }
+                switch(item_type) {
+                    case AS_CDT_CTX_LIST_VALUE:
+                        as_cdt_ctx_add_list_value(cdt_ctx, val);
+                        break;
+                    case AS_CDT_CTX_MAP_KEY:
+                        as_cdt_ctx_add_map_key(cdt_ctx, val);
+                        break;
+                    case AS_CDT_CTX_MAP_VALUE:
+                        as_cdt_ctx_add_map_value(cdt_ctx, val);
+                        break;
+                    default:
+						as_cdt_ctx_destroy(cdt_ctx);
+                        return as_error_update(err, AEROSPIKE_ERR_PARAM, "Failed to convert, unknown ctx operation %s", CTX_KEY);
+                }
+            }
+
+            Py_DECREF(id_temp);
+            Py_DECREF(value_temp);
+        }
+    }
+    else {
+        return as_error_update(err, AEROSPIKE_ERR_PARAM, "Failed to convert %s", CTX_KEY);
+    }
+
+	*ctx_in_use = true;
+	return AEROSPIKE_OK;
+}
+
+bool
+requires_int(uint64_t op) {
+    return(
+        op == AS_CDT_CTX_LIST_INDEX ||
+        op == AS_CDT_CTX_LIST_RANK  ||
+        op == AS_CDT_CTX_MAP_INDEX  ||
+        op == AS_CDT_CTX_MAP_RANK
+    );
 }
\ No newline at end of file
