@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2017 Aerospike, Inc.
+ * Copyright 2013-2019 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -40,25 +40,25 @@ PyObject * AerospikeException_New(void)
 		{&exceptions_array.InvalidRequest, &exceptions_array.ServerFull, &exceptions_array.AlwaysForbidden,
 			&exceptions_array.UnsupportedFeature, &exceptions_array.DeviceOverload, &exceptions_array.NamespaceNotFound, 
 			&exceptions_array.ForbiddenError, &exceptions_array.QueryError, &exceptions_array.ClusterError, 
-			&exceptions_array.InvalidGeoJSON},
+			&exceptions_array.InvalidGeoJSON, &exceptions_array.OpNotApplicable, &exceptions_array.FilteredOut},
 		{"InvalidRequest", "ServerFull", "AlwaysForbidden", "UnsupportedFeature", "DeviceOverload", "NamespaceNotFound",
-			"ForbiddenError", "QueryError", "ClusterError", "InvalidGeoJSON"},
+			"ForbiddenError", "QueryError", "ClusterError", "InvalidGeoJSON", "OpNotApplicable", "FilteredOut"},
 		{AEROSPIKE_ERR_REQUEST_INVALID, AEROSPIKE_ERR_SERVER_FULL, AEROSPIKE_ERR_ALWAYS_FORBIDDEN,
 			AEROSPIKE_ERR_UNSUPPORTED_FEATURE, AEROSPIKE_ERR_DEVICE_OVERLOAD, AEROSPIKE_ERR_NAMESPACE_NOT_FOUND,  
-			AEROSPIKE_ERR_FAIL_FORBIDDEN, AEROSPIKE_ERR_QUERY, AEROSPIKE_ERR_CLUSTER, AEROSPIKE_ERR_GEO_INVALID_GEOJSON}
+			AEROSPIKE_ERR_FAIL_FORBIDDEN, AEROSPIKE_ERR_QUERY, AEROSPIKE_ERR_CLUSTER, AEROSPIKE_ERR_GEO_INVALID_GEOJSON,
+			AEROSPIKE_ERR_OP_NOT_APPLICABLE, AEROSPIKE_FILTERED_OUT}
 	};
 
 	struct record_exceptions_struct record_array = { 
 		{&exceptions_array.RecordKeyMismatch, &exceptions_array.RecordNotFound, &exceptions_array.RecordGenerationError, 
 			&exceptions_array.RecordExistsError, &exceptions_array.RecordTooBig, &exceptions_array.RecordBusy, 
-			&exceptions_array.BinNameError, &exceptions_array.BinExistsError, &exceptions_array.BinNotFound, 
-			&exceptions_array.BinIncompatibleType},
+			&exceptions_array.BinNameError,	&exceptions_array.BinIncompatibleType,
+            &exceptions_array.BinExistsError, &exceptions_array.BinNotFound},
 		{"RecordKeyMismatch", "RecordNotFound", "RecordGenerationError", "RecordExistsError", "RecordTooBig", "RecordBusy", 
-			"BinNameError", "BinExistsError", "BinNotFound", "BinIncompatibleType"},
+			"BinNameError",  "BinIncompatibleType", "BinExistsError", "BinNotFound"},
 		{AEROSPIKE_ERR_RECORD_KEY_MISMATCH, AEROSPIKE_ERR_RECORD_NOT_FOUND, AEROSPIKE_ERR_RECORD_GENERATION, 
 			AEROSPIKE_ERR_RECORD_EXISTS, AEROSPIKE_ERR_RECORD_TOO_BIG, AEROSPIKE_ERR_RECORD_BUSY, 
-			AEROSPIKE_ERR_BIN_NAME, AEROSPIKE_ERR_BIN_EXISTS, AEROSPIKE_ERR_BIN_NOT_FOUND, 
-			AEROSPIKE_ERR_BIN_INCOMPATIBLE_TYPE}
+			AEROSPIKE_ERR_BIN_NAME, AEROSPIKE_ERR_BIN_INCOMPATIBLE_TYPE, AEROSPIKE_ERR_BIN_EXISTS, AEROSPIKE_ERR_BIN_NOT_FOUND}
 	};
 
 	struct index_exceptions_struct index_array = { 
@@ -382,6 +382,7 @@ PyObject* raise_exception(as_error *err) {
 	PyObject * py_key = NULL, *py_value = NULL;
 	Py_ssize_t pos = 0;
 	PyObject * py_module_dict = PyModule_GetDict(module);
+	bool found = false;
 
 	while (PyDict_Next(py_module_dict, &pos, &py_key, &py_value)) {
 		if (PyObject_HasAttrString(py_value, "code")) {
@@ -390,6 +391,7 @@ PyObject* raise_exception(as_error *err) {
 				continue;
 			}
 			if (err->code == PyInt_AsLong(py_code)) {
+				found = true;
 				PyObject *py_attr = NULL;
 				py_attr = PyString_FromString(err->message);
 				PyObject_SetAttrString(py_value, "msg", py_attr);
@@ -420,6 +422,12 @@ PyObject* raise_exception(as_error *err) {
 			Py_DECREF(py_code);
 		}
 	}
-
+    // We haven't found the right exception, just use AerospikeError
+	if (!found) {
+		PyObject* base_exception = PyDict_GetItemString(py_module_dict, "AerospikeError");
+		if (base_exception) {
+			py_value = base_exception;
+		}
+	}
 	return py_value;
 }
