@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2017 Aerospike, Inc.
+ * Copyright 2013-2019 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,12 +24,17 @@
 #include <aerospike/as_admin.h>
 #include <aerospike/as_operations.h>
 #include <aerospike/aerospike_index.h>
+#include <aerospike/as_predexp.h>
 #include "aerospike/as_scan.h"
 #include "aerospike/as_job.h"
 
+#include "conversions.h"
 #include "policy.h"
 #include "macros.h"
 
+#define MAP_WRITE_FLAGS_KEY "map_write_flags"
+#define BIT_WRITE_FLAGS_KEY "bit_write_flags"
+
 #define POLICY_INIT(__policy) \
 	as_error_reset(err);\
 if (!py_policy || py_policy == Py_None) {\
@@ -67,6 +72,19 @@ __policy##_init(policy);\
 	}\
 }
 
+#define POLICY_SET_PREDEXP_BASE_FIELD() {\
+	if (predexp_list) {\
+		PyObject* py_predexp_list = PyDict_GetItemString(py_policy, "predexp");\
+		if (py_predexp_list) {\
+			long number_predexp = PyList_Size(py_predexp_list);\
+			as_predexp_list_init(predexp_list, number_predexp);\
+			convert_predexp_list(py_predexp_list, predexp_list, err);\
+			policy->base.predexp = predexp_list;\
+			*predexp_list_p = predexp_list;\
+		}\
+	}\
+}
+
 /* This allows the old timeout field to properly set timeout, remove in future release: > 3.1 */
 #define POLICY_SET_TOTAL_TIMEOUT_FROM_TIMEOUT() { \
 	PyObject * py_field = PyDict_GetItemString(py_policy, "timeout");\
@@ -126,8 +144,7 @@ AerospikeConstants aerospike_constants[] = {
 	{ AS_POLICY_REPLICA_MASTER              ,   "POLICY_REPLICA_MASTER" },
 	{ AS_POLICY_REPLICA_ANY                 ,   "POLICY_REPLICA_ANY" },
 	{ AS_POLICY_REPLICA_SEQUENCE            ,   "POLICY_REPLICA_SEQUENCE" },
-	{ AS_POLICY_CONSISTENCY_LEVEL_ONE       ,   "POLICY_CONSISTENCY_ONE" },
-	{ AS_POLICY_CONSISTENCY_LEVEL_ALL       ,   "POLICY_CONSISTENCY_ALL" },
+	{ AS_POLICY_REPLICA_PREFER_RACK         ,   "POLICY_REPLICA_PREFER_RACK" },
 	{ AS_POLICY_COMMIT_LEVEL_ALL            ,   "POLICY_COMMIT_LEVEL_ALL" },
 	{ AS_POLICY_COMMIT_LEVEL_MASTER         ,   "POLICY_COMMIT_LEVEL_MASTER" },
 	{ SERIALIZER_PYTHON                     ,   "SERIALIZER_PYTHON" },
@@ -144,6 +161,7 @@ AerospikeConstants aerospike_constants[] = {
 	{ AS_PRIVILEGE_SYS_ADMIN                ,   "PRIV_SYS_ADMIN"	},
 	{ AS_PRIVILEGE_DATA_ADMIN               ,   "PRIV_DATA_ADMIN"	},
 	{ AS_PRIVILEGE_READ                     ,   "PRIV_READ"},
+	{ AS_PRIVILEGE_WRITE                     ,   "PRIV_WRITE"},
 	{ AS_PRIVILEGE_READ_WRITE               ,   "PRIV_READ_WRITE"},
 	{ AS_PRIVILEGE_READ_WRITE_UDF           ,   "PRIV_READ_WRITE_UDF"},
 
@@ -256,7 +274,91 @@ AerospikeConstants aerospike_constants[] = {
 	{ AS_MAP_WRITE_PARTIAL, "MAP_WRITE_PARTIAL"},
 
 	{ AS_LIST_WRITE_NO_FAIL, "LIST_WRITE_NO_FAIL"},
-	{ AS_LIST_WRITE_PARTIAL, "LIST_WRITE_PARTIAL"}
+	{ AS_LIST_WRITE_PARTIAL, "LIST_WRITE_PARTIAL"},
+
+	/* Map write flags post 3.5.0 */
+	{ AS_MAP_WRITE_DEFAULT, "MAP_WRITE_FLAGS_DEFAULT"},
+	{ AS_MAP_WRITE_CREATE_ONLY, "MAP_WRITE_FLAGS_CREATE_ONLY"},
+	{ AS_MAP_WRITE_UPDATE_ONLY, "MAP_WRITE_FLAGS_UPDATE_ONLY"},
+	{ AS_MAP_WRITE_NO_FAIL, "MAP_WRITE_FLAGS_NO_FAIL"},
+	{ AS_MAP_WRITE_PARTIAL, "MAP_WRITE_FLAGS_PARTIAL"},
+
+	/* READ Mode constants 4.0.0 */
+
+	// AP Read Mode
+	{ AS_POLICY_READ_MODE_AP_ONE, "POLICY_READ_MODE_AP_ONE"},
+	{ AS_POLICY_READ_MODE_AP_ALL, "POLICY_READ_MODE_AP_ALL"},
+	// SC Read Mode
+	{ AS_POLICY_READ_MODE_SC_SESSION, "POLICY_READ_MODE_SC_SESSION" },
+	{ AS_POLICY_READ_MODE_SC_LINEARIZE, "POLICY_READ_MODE_SC_LINEARIZE"},
+	{ AS_POLICY_READ_MODE_SC_ALLOW_REPLICA, "POLICY_READ_MODE_SC_ALLOW_REPLICA"},
+	{ AS_POLICY_READ_MODE_SC_ALLOW_UNAVAILABLE, "POLICY_READ_MODE_SC_ALLOW_UNAVAILABLE"},
+
+	/* Bitwise constants: 3.9.0 */
+	{ AS_BIT_WRITE_DEFAULT, "BIT_WRITE_DEFAULT"},
+	{ AS_BIT_WRITE_CREATE_ONLY, "BIT_WRITE_CREATE_ONLY"},
+	{ AS_BIT_WRITE_UPDATE_ONLY, "BIT_WRITE_UPDATE_ONLY"},
+	{ AS_BIT_WRITE_NO_FAIL, "BIT_WRITE_NO_FAIL"},
+	{ AS_BIT_WRITE_PARTIAL, "BIT_WRITE_PARTIAL"},
+
+	{ AS_BIT_RESIZE_DEFAULT, "BIT_RESIZE_DEFAULT"},
+	{ AS_BIT_RESIZE_FROM_FRONT, "BIT_RESIZE_FROM_FRONT"},
+	{ AS_BIT_RESIZE_GROW_ONLY, "BIT_RESIZE_GROW_ONLY"},
+	{ AS_BIT_RESIZE_SHRINK_ONLY, "BIT_RESIZE_SHRINK_ONLY"},
+
+	{ AS_BIT_OVERFLOW_FAIL, "BIT_OVERFLOW_FAIL"},
+	{ AS_BIT_OVERFLOW_SATURATE, "BIT_OVERFLOW_SATURATE"},
+	{ AS_BIT_OVERFLOW_WRAP, "BIT_OVERFLOW_WRAP"},
+
+    /* BITWISE OPS: 3.9.0 */
+	{ OP_BIT_INSERT, "OP_BIT_INSERT"},
+	{ OP_BIT_RESIZE, "OP_BIT_RESIZE"},
+	{ OP_BIT_REMOVE, "OP_BIT_REMOVE"},
+	{ OP_BIT_SET, "OP_BIT_SET"},
+	{ OP_BIT_OR, "OP_BIT_OR"},
+	{ OP_BIT_XOR, "OP_BIT_XOR"},
+	{ OP_BIT_AND, "OP_BIT_AND"},
+	{ OP_BIT_NOT, "OP_BIT_NOT"},
+	{ OP_BIT_LSHIFT, "OP_BIT_LSHIFT"},
+	{ OP_BIT_RSHIFT, "OP_BIT_RSHIFT"},
+	{ OP_BIT_ADD, "OP_BIT_ADD" },
+	{ OP_BIT_SUBTRACT, "OP_BIT_SUBTRACT"},
+	{ OP_BIT_GET_INT, "OP_BIT_GET_INT"},
+	{ OP_BIT_SET_INT, "OP_BIT_SET_INT"},
+	{ OP_BIT_GET, "OP_BIT_GET"},
+	{ OP_BIT_COUNT, "OP_BIT_COUNT"},
+	{ OP_BIT_LSCAN, "OP_BIT_LSCAN"},
+	{ OP_BIT_RSCAN, "OP_BIT_RSCAN"},
+
+	/* Nested CDT constants: 3.9.0 */
+	{ AS_CDT_CTX_LIST_INDEX, "CDT_CTX_LIST_INDEX"},
+	{ AS_CDT_CTX_LIST_RANK, "CDT_CTX_LIST_RANK"},
+	{ AS_CDT_CTX_LIST_VALUE, "CDT_CTX_LIST_VALUE"},
+	{ AS_CDT_CTX_MAP_INDEX, "CDT_CTX_MAP_INDEX"},
+	{ AS_CDT_CTX_MAP_RANK, "CDT_CTX_MAP_RANK"},
+	{ AS_CDT_CTX_MAP_KEY, "CDT_CTX_MAP_KEY"},
+	{ AS_CDT_CTX_MAP_VALUE, "CDT_CTX_MAP_VALUE"},
+
+	/* HLL constants 3.11.0 */
+	{ OP_HLL_ADD, "OP_HLL_ADD"},
+	{ OP_HLL_DESCRIBE, "OP_HLL_DESCRIBE"},
+	{ OP_HLL_FOLD, "OP_HLL_FOLD"},
+	{ OP_HLL_GET_COUNT, "OP_HLL_GET_COUNT"},
+	{ OP_HLL_GET_INTERSECT_COUNT, "OP_HLL_GET_INTERSECT_COUNT"},
+	{ OP_HLL_GET_SIMILARITY, "OP_HLL_GET_SIMILARITY"},
+	{ OP_HLL_GET_UNION, "OP_HLL_GET_UNION"},
+	{ OP_HLL_GET_UNION_COUNT, "OP_HLL_GET_UNION_COUNT"},
+	{ OP_HLL_GET_SIMILARITY, "OP_HLL_GET_SIMILARITY"},
+	{ OP_HLL_INIT, "OP_HLL_INIT"},
+	{ OP_HLL_REFRESH_COUNT, "OP_HLL_REFRESH_COUNT"},
+	{ OP_HLL_SET_UNION, "OP_HLL_SET_UNION"},
+
+	{ AS_HLL_WRITE_DEFAULT, "HLL_WRITE_DEFAULT"},
+	{ AS_HLL_WRITE_CREATE_ONLY, "HLL_WRITE_CREATE_ONLY"},
+	{ AS_HLL_WRITE_UPDATE_ONLY, "HLL_WRITE_UPDATE_ONLY"},
+	{ AS_HLL_WRITE_NO_FAIL, "HLL_WRITE_NO_FAIL"},
+	{ AS_HLL_WRITE_ALLOW_FOLD, "HLL_WRITE_ALLOW_FOLD"},
+
 };
 
 static
@@ -428,7 +530,9 @@ as_status pyobject_to_policy_admin(as_error * err, PyObject * py_policy,
 as_status pyobject_to_policy_apply(as_error * err, PyObject * py_policy,
 		as_policy_apply * policy,
 		as_policy_apply ** policy_p,
-		as_policy_apply * config_apply_policy)
+		as_policy_apply * config_apply_policy,
+		as_predexp_list * predexp_list,
+		as_predexp_list ** predexp_list_p)
 {
 	// Initialize Policy
 	POLICY_INIT(as_policy_apply);
@@ -443,13 +547,16 @@ as_status pyobject_to_policy_apply(as_error * err, PyObject * py_policy,
 	POLICY_SET_BASE_FIELD(socket_timeout, uint32_t);
 	POLICY_SET_BASE_FIELD(max_retries, uint32_t);
 	POLICY_SET_BASE_FIELD(sleep_between_retries, uint32_t);
+	POLICY_SET_BASE_FIELD(compress, bool);
 
 	POLICY_SET_FIELD(key, as_policy_key);
 	POLICY_SET_FIELD(replica, as_policy_replica);
 	POLICY_SET_FIELD(gen, as_policy_gen);
 	POLICY_SET_FIELD(commit_level, as_policy_commit_level);
 	POLICY_SET_FIELD(durable_delete, bool);
-	POLICY_SET_FIELD(linearize_read, bool);
+
+	// C client 4.6.7 new policy
+	POLICY_SET_PREDEXP_BASE_FIELD();
 
 	// Update the policy
 	POLICY_UPDATE();
@@ -490,11 +597,14 @@ as_status pyobject_to_policy_info(as_error * err, PyObject * py_policy,
  * Returns AEROSPIKE_OK on success. On error, the err argument is populated.
  * We assume that the error object and the policy object are already allocated
  * and initialized (although, we do reset the error object here).
+ * predexp_list is initialized by this function, caller must free.
  */
 as_status pyobject_to_policy_query(as_error * err, PyObject * py_policy,
 		as_policy_query * policy,
 		as_policy_query ** policy_p,
-		as_policy_query * config_query_policy)
+		as_policy_query * config_query_policy,
+		as_predexp_list * predexp_list,
+		as_predexp_list ** predexp_list_p)
 {
 	// Initialize Policy
 	POLICY_INIT(as_policy_query);
@@ -508,10 +618,15 @@ as_status pyobject_to_policy_query(as_error * err, PyObject * py_policy,
 	POLICY_SET_BASE_FIELD(socket_timeout, uint32_t);
 	POLICY_SET_BASE_FIELD(max_retries, uint32_t);
 	POLICY_SET_BASE_FIELD(sleep_between_retries, uint32_t);
+	POLICY_SET_BASE_FIELD(compress, bool);
 
 
 	POLICY_SET_FIELD(deserialize, bool);
 	POLICY_SET_FIELD(fail_on_cluster_change, bool);
+
+	// C client 4.6.7 new policy
+	POLICY_SET_PREDEXP_BASE_FIELD();
+
 	// Update the policy
 	POLICY_UPDATE();
 
@@ -527,7 +642,9 @@ as_status pyobject_to_policy_query(as_error * err, PyObject * py_policy,
 as_status pyobject_to_policy_read(as_error * err, PyObject * py_policy,
 		as_policy_read * policy,
 		as_policy_read ** policy_p,
-		as_policy_read * config_read_policy)
+		as_policy_read * config_read_policy,
+		as_predexp_list * predexp_list,
+		as_predexp_list ** predexp_list_p)
 {
 	// Initialize Policy
 	POLICY_INIT(as_policy_read);
@@ -542,12 +659,18 @@ as_status pyobject_to_policy_read(as_error * err, PyObject * py_policy,
 	POLICY_SET_BASE_FIELD(socket_timeout, uint32_t);
 	POLICY_SET_BASE_FIELD(max_retries, uint32_t);
 	POLICY_SET_BASE_FIELD(sleep_between_retries, uint32_t);
+	POLICY_SET_BASE_FIELD(compress, bool);
 
 	POLICY_SET_FIELD(key, as_policy_key);
-	POLICY_SET_FIELD(consistency_level, as_policy_consistency_level);
 	POLICY_SET_FIELD(replica, as_policy_replica);
 	POLICY_SET_FIELD(deserialize, bool);
-	POLICY_SET_FIELD(linearize_read, bool);
+
+	// 4.0.0 new policies
+	POLICY_SET_FIELD(read_mode_ap, as_policy_read_mode_ap);
+	POLICY_SET_FIELD(read_mode_sc, as_policy_read_mode_sc);
+
+	// C client 4.6.7 new policy
+	POLICY_SET_PREDEXP_BASE_FIELD();
 
 	// Update the policy
 	POLICY_UPDATE();
@@ -564,7 +687,9 @@ as_status pyobject_to_policy_read(as_error * err, PyObject * py_policy,
 as_status pyobject_to_policy_remove(as_error * err, PyObject * py_policy,
 		as_policy_remove * policy,
 		as_policy_remove ** policy_p,
-		as_policy_remove * config_remove_policy)
+		as_policy_remove * config_remove_policy,
+		as_predexp_list * predexp_list,
+		as_predexp_list ** predexp_list_p)
 {
 	// Initialize Policy
 	POLICY_INIT(as_policy_remove);
@@ -579,6 +704,7 @@ as_status pyobject_to_policy_remove(as_error * err, PyObject * py_policy,
 	POLICY_SET_BASE_FIELD(socket_timeout, uint32_t);
 	POLICY_SET_BASE_FIELD(max_retries, uint32_t);
 	POLICY_SET_BASE_FIELD(sleep_between_retries, uint32_t);
+	POLICY_SET_BASE_FIELD(compress, bool);
 
 	POLICY_SET_FIELD(generation, uint16_t);
 
@@ -588,6 +714,9 @@ as_status pyobject_to_policy_remove(as_error * err, PyObject * py_policy,
 	POLICY_SET_FIELD(replica, as_policy_replica);
 	POLICY_SET_FIELD(durable_delete, bool);
 
+	// C client 4.6.7 new policy
+	POLICY_SET_PREDEXP_BASE_FIELD();
+
 	// Update the policy
 	POLICY_UPDATE();
 
@@ -603,7 +732,9 @@ as_status pyobject_to_policy_remove(as_error * err, PyObject * py_policy,
 as_status pyobject_to_policy_scan(as_error * err, PyObject * py_policy,
 		as_policy_scan * policy,
 		as_policy_scan ** policy_p,
-		as_policy_scan * config_scan_policy)
+		as_policy_scan * config_scan_policy,
+		as_predexp_list * predexp_list,
+		as_predexp_list ** predexp_list_p)
 {
 	// Initialize Policy
 	POLICY_INIT(as_policy_scan);
@@ -619,9 +750,14 @@ as_status pyobject_to_policy_scan(as_error * err, PyObject * py_policy,
 	POLICY_SET_BASE_FIELD(socket_timeout, uint32_t);
 	POLICY_SET_BASE_FIELD(max_retries, uint32_t);
 	POLICY_SET_BASE_FIELD(sleep_between_retries, uint32_t);
+	POLICY_SET_BASE_FIELD(compress, bool);
 
 	POLICY_SET_FIELD(fail_on_cluster_change, bool);
 	POLICY_SET_FIELD(durable_delete, bool);
+	POLICY_SET_FIELD(records_per_second, uint32_t);
+
+	// C client 4.6.7 new policy
+	POLICY_SET_PREDEXP_BASE_FIELD();
 
 	// Update the policy
 	POLICY_UPDATE();
@@ -638,7 +774,9 @@ as_status pyobject_to_policy_scan(as_error * err, PyObject * py_policy,
 as_status pyobject_to_policy_write(as_error * err, PyObject * py_policy,
 		as_policy_write * policy,
 		as_policy_write ** policy_p,
-		as_policy_write * config_write_policy)
+		as_policy_write * config_write_policy,
+		as_predexp_list * predexp_list,
+		as_predexp_list ** predexp_list_p)
 {
 	// Initialize Policy
 	POLICY_INIT(as_policy_write);
@@ -654,6 +792,7 @@ as_status pyobject_to_policy_write(as_error * err, PyObject * py_policy,
 	POLICY_SET_BASE_FIELD(socket_timeout, uint32_t);
 	POLICY_SET_BASE_FIELD(max_retries, uint32_t);
 	POLICY_SET_BASE_FIELD(sleep_between_retries, uint32_t);
+	POLICY_SET_BASE_FIELD(compress, bool);
 
 	POLICY_SET_FIELD(key, as_policy_key);
 	POLICY_SET_FIELD(gen, as_policy_gen);
@@ -663,6 +802,9 @@ as_status pyobject_to_policy_write(as_error * err, PyObject * py_policy,
 	POLICY_SET_FIELD(replica, as_policy_replica);
 	POLICY_SET_FIELD(compression_threshold, uint32_t);
 
+	// C client 4.6.7 new policy
+	POLICY_SET_PREDEXP_BASE_FIELD();
+
 	// Update the policy
 	POLICY_UPDATE();
 
@@ -678,7 +820,9 @@ as_status pyobject_to_policy_write(as_error * err, PyObject * py_policy,
 as_status pyobject_to_policy_operate(as_error * err, PyObject * py_policy,
 		as_policy_operate * policy,
 		as_policy_operate ** policy_p,
-		as_policy_operate * config_operate_policy)
+		as_policy_operate * config_operate_policy,
+		as_predexp_list * predexp_list,
+		as_predexp_list ** predexp_list_p)
 {
 	// Initialize Policy
 	POLICY_INIT(as_policy_operate);
@@ -693,17 +837,23 @@ as_status pyobject_to_policy_operate(as_error * err, PyObject * py_policy,
 	POLICY_SET_BASE_FIELD(socket_timeout, uint32_t);
 	POLICY_SET_BASE_FIELD(max_retries, uint32_t);
 	POLICY_SET_BASE_FIELD(sleep_between_retries, uint32_t);
+	POLICY_SET_BASE_FIELD(compress, bool);
 
 	POLICY_SET_FIELD(key, as_policy_key);
 	POLICY_SET_FIELD(gen, as_policy_gen);
 	POLICY_SET_FIELD(commit_level, as_policy_commit_level);
-	POLICY_SET_FIELD(consistency_level, as_policy_consistency_level);
 	POLICY_SET_FIELD(replica, as_policy_replica);
 	POLICY_SET_FIELD(durable_delete, bool);
 	POLICY_SET_FIELD(deserialize, bool);
-	POLICY_SET_FIELD(linearize_read, bool);
 	POLICY_SET_FIELD(exists, as_policy_exists);
 
+	// 4.0.0 new policies
+	POLICY_SET_FIELD(read_mode_ap, as_policy_read_mode_ap);
+	POLICY_SET_FIELD(read_mode_sc, as_policy_read_mode_sc);
+
+	// C client 4.6.7 new policy
+	POLICY_SET_PREDEXP_BASE_FIELD();
+
 	// Update the policy
 	POLICY_UPDATE();
 
@@ -719,7 +869,9 @@ as_status pyobject_to_policy_operate(as_error * err, PyObject * py_policy,
 as_status pyobject_to_policy_batch(as_error * err, PyObject * py_policy,
 		as_policy_batch * policy,
 		as_policy_batch ** policy_p,
-		as_policy_batch * config_batch_policy)
+		as_policy_batch * config_batch_policy,
+		as_predexp_list * predexp_list,
+		as_predexp_list ** predexp_list_p)
 {
 	// Initialize Policy
 	POLICY_INIT(as_policy_batch);
@@ -734,21 +886,45 @@ as_status pyobject_to_policy_batch(as_error * err, PyObject * py_policy,
 	POLICY_SET_BASE_FIELD(socket_timeout, uint32_t);
 	POLICY_SET_BASE_FIELD(max_retries, uint32_t);
 	POLICY_SET_BASE_FIELD(sleep_between_retries, uint32_t);
+	POLICY_SET_BASE_FIELD(compress, bool);
 
-	POLICY_SET_FIELD(consistency_level, as_policy_consistency_level);
 	POLICY_SET_FIELD(concurrent, bool);
-	POLICY_SET_FIELD(use_batch_direct, bool);
 	POLICY_SET_FIELD(allow_inline, bool);
 	POLICY_SET_FIELD(send_set_name, bool);
 	POLICY_SET_FIELD(deserialize, bool);
-	POLICY_SET_FIELD(linearize_read, bool);
+	POLICY_SET_FIELD(replica, as_policy_replica);
+
+	// 4.0.0 new policies
+	POLICY_SET_FIELD(read_mode_ap, as_policy_read_mode_ap);
+	POLICY_SET_FIELD(read_mode_sc, as_policy_read_mode_sc);
+
+	// C client 4.6.7 new policy
+	POLICY_SET_PREDEXP_BASE_FIELD();
 
 	// Update the policy
 	POLICY_UPDATE();
 
 	return err->code;
 }
+as_status
+pyobject_to_bit_policy(as_error* err, PyObject* py_policy, as_bit_policy* policy) {
+    as_bit_policy_init(policy);
+    POLICY_INIT(as_bit_policy);
 
+    PyObject* py_bit_flags = PyDict_GetItemString(py_policy, BIT_WRITE_FLAGS_KEY);
+    if (py_bit_flags) {
+        if (PyInt_Check(py_bit_flags)){
+        	as_bit_write_flags bit_write_flags = (as_bit_write_flags)PyInt_AsLong(py_bit_flags);
+            as_bit_policy_set_write_flags(policy, bit_write_flags);
+        }
+    } else if (PyErr_Occurred()) {
+        /* Fetching a map key failed internally for some reason, raise an error and exit.*/
+        PyErr_Clear();
+        return as_error_update(err, AEROSPIKE_ERR_CLIENT, "Unable to get bit_write_flags");
+    }
+    
+    return err->code;
+}
 as_status pyobject_to_map_policy(as_error * err, PyObject * py_policy,
 		as_map_policy * policy)
 {
@@ -757,10 +933,27 @@ as_status pyobject_to_map_policy(as_error * err, PyObject * py_policy,
 
 	long map_order = AS_MAP_UNORDERED;
 	long map_write_mode = AS_MAP_UPDATE;
+	uint32_t map_write_flags = AS_MAP_WRITE_DEFAULT;
+
+	MAP_POLICY_SET_FIELD(map_order);
+	PyObject* mode_or_flags = PyDict_GetItemString(py_policy, MAP_WRITE_FLAGS_KEY);
+
+	/*
+	This only works for client >= 3.5.0 and server >= 4.3.0
+	If py_policy["map_write_flags"] is set, we use it
+	otherwise we use py_policy["map_write_mode"]
+	*/
+	if (mode_or_flags) {
+		if (PyInt_Check(mode_or_flags)) {
+			map_write_flags = (uint32_t)PyInt_AsLong(mode_or_flags);
+			as_map_policy_set_flags(policy, map_order, map_write_flags);
+		} else {
+			as_error_update(err, AEROSPIKE_ERR_PARAM, "map write flags must be an integer");
+		}
+		return err->code;
+	}
 
 	MAP_POLICY_SET_FIELD(map_write_mode);
-	MAP_POLICY_SET_FIELD(map_order);
-
 	as_map_policy_set(policy, map_order, map_write_mode);
 
 	return err->code;
@@ -826,3 +1019,41 @@ pyobject_to_list_policy(as_error* err, PyObject* py_policy, as_list_policy* list
 
 	return AEROSPIKE_OK;
 }
+
+as_status
+pyobject_to_hll_policy(as_error* err, PyObject* py_policy, as_hll_policy* hll_policy) {
+	int64_t flags = 0;
+	as_hll_policy_init(hll_policy);
+	PyObject* py_val = NULL;
+
+	if (!py_policy || py_policy == Py_None) {
+		return AEROSPIKE_OK;
+	}
+
+	if (!PyDict_Check(py_policy)) {
+		return as_error_update(err, AEROSPIKE_ERR_PARAM, "Hll policy must be a dictionary.");
+	}
+
+	py_val = PyDict_GetItemString(py_policy, "flags");
+	if (py_val && py_val != Py_None) {
+		if (PyInt_Check(py_val)) {
+			flags = (int64_t)PyInt_AsLong(py_val);
+			if (PyErr_Occurred()) {
+				return as_error_update(err, AEROSPIKE_ERR_PARAM, "Failed to convert flags.");
+			}
+		}
+		else if (PyLong_Check(py_val)) {
+			flags = (int64_t)PyLong_AsLong(py_val);
+			if (PyErr_Occurred()) {
+				return as_error_update(err, AEROSPIKE_ERR_PARAM, "Failed to convert flags.");
+			}
+		}
+		else {
+			return as_error_update(err, AEROSPIKE_ERR_PARAM, "Invalid hll policy flags.");
+		}
+	}
+
+	as_hll_policy_set_write_flags(hll_policy, flags);
+
+	return AEROSPIKE_OK;
+}
