@@ -131,6 +131,10 @@ PyObject * AerospikeQuery_Foreach(AerospikeQuery * self, PyObject * args, PyObje
 	as_policy_query query_policy;
 	as_policy_query * query_policy_p = NULL;
 
+	// For converting predexp.
+	as_predexp_list predexp_list;
+	as_predexp_list* predexp_list_p = NULL;
+
 	// Initialize error
 	as_error_init(&err);
 
@@ -146,7 +150,7 @@ PyObject * AerospikeQuery_Foreach(AerospikeQuery * self, PyObject * args, PyObje
 
 	// Convert python policy object to as_policy_exists
 	pyobject_to_policy_query(&err, py_policy, &query_policy, &query_policy_p,
-			&self->client->as->config.policies.query);
+			&self->client->as->config.policies.query, &predexp_list, &predexp_list_p);
 	if (err.code != AEROSPIKE_OK) {
 		goto CLEANUP;
 	}
@@ -169,6 +173,10 @@ PyObject * AerospikeQuery_Foreach(AerospikeQuery * self, PyObject * args, PyObje
 	}
 
 CLEANUP:
+	if (predexp_list_p) {
+		as_predexp_list_destroy(&predexp_list);
+	}
+
 	if (self->query.apply.arglist) {
 		as_arraylist_destroy( (as_arraylist *) self->query.apply.arglist );
 	}
