@@ -50,30 +50,18 @@ static bool each_result(const as_val * val, void * udata)
 
 	as_error err;
 
-	TRACE();
-
 	PyGILState_STATE gstate;
 	gstate = PyGILState_Ensure();
 
-	TRACE();
-
 	val_to_pyobject(data->client, &err, val, &py_result);
 
-	TRACE();
 
 	if (py_result) {
-		TRACE();
 		PyList_Append(py_results, py_result);
-
-		TRACE();
 		Py_DECREF(py_result);
 	}
-
-	TRACE();
-
 	PyGILState_Release(gstate);
 
-	TRACE();
 	return true;
 }
 
@@ -98,6 +86,10 @@ PyObject * AerospikeQuery_Results(AerospikeQuery * self, PyObject * args, PyObje
 	as_policy_query query_policy;
 	as_policy_query * query_policy_p = NULL;
 
+	// For converting predexp.
+	as_predexp_list predexp_list;
+	as_predexp_list* predexp_list_p = NULL;
+
 	if (!self || !self->client->as) {
 		as_error_update(&err, AEROSPIKE_ERR_PARAM, "Invalid aerospike object");
 		goto CLEANUP;
@@ -110,7 +102,7 @@ PyObject * AerospikeQuery_Results(AerospikeQuery * self, PyObject * args, PyObje
 
 	// Convert python policy object to as_policy_query
 	pyobject_to_policy_query(&err, py_policy, &query_policy, &query_policy_p,
-			&self->client->as->config.policies.query);
+			&self->client->as->config.policies.query, &predexp_list, &predexp_list_p);
 	if (err.code != AEROSPIKE_OK) {
 		goto CLEANUP;
 	}
@@ -119,33 +111,30 @@ PyObject * AerospikeQuery_Results(AerospikeQuery * self, PyObject * args, PyObje
 		goto CLEANUP;
 	}
 
-	TRACE();
 	py_results = PyList_New(0);
 	data.py_results = py_results;
 
-	TRACE();
 	PyThreadState * _save = PyEval_SaveThread();
 
-	TRACE();
 	aerospike_query_foreach(self->client->as, &err, query_policy_p, &self->query, each_result, &data);
 
-	TRACE();
 	PyEval_RestoreThread(_save);
 
 CLEANUP:/*??trace()*/
-	TRACE();
+	if (predexp_list_p) {
+		as_predexp_list_destroy(&predexp_list);
+	}
+
 	if (err.code != AEROSPIKE_OK) {
+		Py_XDECREF(py_results);
 		PyObject * py_err = NULL;
 		error_to_pyobject(&err, &py_err);
 		PyObject *exception_type = raise_exception(&err);
 		PyErr_SetObject(exception_type, py_err);
 		Py_DECREF(py_err);
-		TRACE();
 		return NULL;
 	}
 
-	TRACE();
-
 	if (self->query.apply.arglist) {
 		as_arraylist_destroy( (as_arraylist *) self->query.apply.arglist );
 	}
