@@ -251,7 +251,7 @@ void execute_user_callback(user_serializer_callback *user_callback_info,
 			char *py_val;
 			Py_ssize_t len;
 #if PY_MAJOR_VERSION >= 3
-			py_val = PyUnicode_AsUTF8AndSize(py_return, &len);
+			py_val = (char*)PyUnicode_AsUTF8AndSize(py_return, &len);
 #else
 			py_val = PyString_AsString(py_return);
 			len = PyString_Size(py_return);
@@ -335,6 +335,10 @@ extern as_status serialize_based_on_serializer_policy(AerospikeClient * self,
 					uint8_t *bytes_array = (uint8_t *) PyByteArray_AsString(value);
 					uint32_t bytes_array_len  = (uint32_t)  PyByteArray_Size(value);
 					set_as_bytes(bytes, bytes_array, bytes_array_len, AS_BYTES_BLOB, error_p);
+				} else if (PyBytes_Check(value)) {
+					uint8_t *my_bytes = (uint8_t *) PyBytes_AsString(value);
+					uint32_t my_bytes_len  = (uint32_t)  PyBytes_Size(value);
+					set_as_bytes(bytes, my_bytes, my_bytes_len, AS_BYTES_BLOB, error_p);
 				} else {
 
 					/* get the sys.modules dictionary */
@@ -533,11 +537,6 @@ extern as_status deserialize_based_on_as_bytes_type(AerospikeClient * self,
 								}
 							}
 							break;
-		case AS_BYTES_LDT:  {
-								Py_INCREF(Py_None);
-								*retval = Py_None;
-							}
-							break;
 		default:			{
 								// First try to return a raw byte array, if that fails raise an error
 								uint32_t bval_size = as_bytes_size(bytes);
